# generated by 'xml2py'
# flags '-c -d -v C:\test_h\PixelType.xml -o PixelType_header.py'
# pylint: skip-file
from ctypes import *


PixelType_Gvsp_YUV411_Packed = 34340894
PixelType_Gvsp_Mono1p = 16842807
PixelType_Gvsp_YCBCR422_8 = 34603067
PixelType_Gvsp_BayerGR10 = 17825804
PixelType_Gvsp_BayerGR12_Packed = 17563690
PixelType_Gvsp_BayerGB8 = 17301514
PixelType_Gvsp_BayerGB16 = 17825840
PixelType_Gvsp_YCBCR601_422_8 = 34603070
PixelType_Gvsp_YCBCR709_8_CBYCR = 35127360
PixelType_Gvsp_BayerRG10 = 17825805
PixelType_Gvsp_RGB10_Packed = 36700184
PixelType_Gvsp_COORD3D_DEPTH_PLUS_MASK = -2112094207
PixelType_Gvsp_YCBCR601_422_8_CBYCRY = 34603076
PixelType_Gvsp_Mono14 = 17825829
PixelType_Gvsp_YCBCR601_411_8_CBYYCRYY = 34340927
PixelType_Gvsp_BayerGB10_Packed = 17563688
PixelType_Gvsp_BayerGB12 = 17825810
PixelType_Gvsp_YUV422_YUYV_Packed = 34603058
PixelType_Gvsp_Mono8 = 17301505
PixelType_Gvsp_BayerGR12 = 17825808
PixelType_Gvsp_YCBCR709_422_8 = 34603073
PixelType_Gvsp_RGB10V1_Packed = 35651612
PixelType_Gvsp_BayerBG12 = 17825811
PixelType_Gvsp_BayerRG12 = 17825809
PixelType_Gvsp_BayerBG10 = 17825807
PixelType_Gvsp_RGB8_Packed = 35127316
PixelType_Gvsp_BGR565_Packed = 34603062
PixelType_Gvsp_Coord3D_ABC32f_Planar = 39846081
PixelType_Gvsp_YCBCR601_8_CBYCR = 35127357
PixelType_Gvsp_BayerGB10 = 17825806
PixelType_Gvsp_YCBCR8_CBYCR = 35127354
PixelType_Gvsp_BayerRG8 = 17301513
PixelType_Gvsp_BayerBG12_Packed = 17563693
PixelType_Gvsp_BayerRG16 = 17825839
PixelType_Gvsp_BayerBG16 = 17825841
PixelType_Gvsp_BayerGR16 = 17825838
PixelType_Gvsp_Undefined = -1
PixelType_Gvsp_BayerBG10_Packed = 17563689
PixelType_Gvsp_BayerGB12_Packed = 17563692
PixelType_Gvsp_BayerBG8 = 17301515
PixelType_Gvsp_YCBCR422_8_CBYCRY = 34603075
PixelType_Gvsp_RGB10_Planar = 36700194
PixelType_Gvsp_BayerGR10_Packed = 17563686
PixelType_Gvsp_RGB16_Planar = 36700196
PixelType_Gvsp_RGB565_Packed = 34603061
PixelType_Gvsp_Mono4p = 17039417
PixelType_Gvsp_Coord3D_ABC32f = 39846080
PixelType_Gvsp_BGR10_Packed = 36700185
PixelType_Gvsp_YCBCR411_8_CBYYCRYY = 34340924
PixelType_Gvsp_BayerRG12_Packed = 17563691
PixelType_Gvsp_YUV422_Packed = 34603039
PixelType_Gvsp_YCBCR709_411_8_CBYYCRYY = 34340930
PixelType_Gvsp_RGB8_Planar = 35127329
PixelType_Gvsp_Mono10 = 17825795
PixelType_Gvsp_BayerGR8 = 17301512
PixelType_Gvsp_YCBCR709_422_8_CBYCRY = 34603077
PixelType_Gvsp_Jpeg = -2145910783
PixelType_Gvsp_RGB12_Packed = 36700186
PixelType_Gvsp_RGB12_Planar = 36700195
PixelType_Gvsp_Mono2p = 16908344
PixelType_Gvsp_BGR12_Packed = 36700187
PixelType_Gvsp_RGB12V1_Packed = 35913780
PixelType_Gvsp_BGR8_Packed = 35127317
PixelType_Gvsp_RGBA8_Packed = 35651606
PixelType_Gvsp_RGB16_Packed = 36700211
PixelType_Gvsp_YUV444_Packed = 35127328
PixelType_Gvsp_RGB10V2_Packed = 35651613
PixelType_Gvsp_Mono16 = 17825799
PixelType_Gvsp_Mono8_Signed = 17301506
PixelType_Gvsp_Mono10_Packed = 17563652
PixelType_Gvsp_BGRA8_Packed = 35651607
PixelType_Gvsp_Mono12_Packed = 17563654
PixelType_Gvsp_Mono12 = 17825797
PixelType_Gvsp_BayerRG10_Packed = 17563687
PixelType_Gvsp_Coord3D_AC32f = 36176066
int8_t = c_int8
int16_t = c_int16
int32_t = c_int32
int64_t = c_int64
uint8_t = c_uint8
uint16_t = c_uint16
uint32_t = c_uint32
uint64_t = c_uint64
int_least8_t = c_byte
int_least16_t = c_short
int_least32_t = c_int
int_least64_t = c_long
uint_least8_t = c_ubyte
uint_least16_t = c_ushort
uint_least32_t = c_uint
uint_least64_t = c_ulong
int_fast8_t = c_byte
int_fast16_t = c_long
int_fast32_t = c_long
int_fast64_t = c_long
uint_fast8_t = c_ubyte
uint_fast16_t = c_ulong
uint_fast32_t = c_ulong
uint_fast64_t = c_ulong
intptr_t = c_long
uintptr_t = c_ulong
intmax_t = c_long
uintmax_t = c_ulong

# values for enumeration 'MvGvspPixelType'
MvGvspPixelType = c_int  # enum
__all__ = [
    "PixelType_Gvsp_BayerRG8",
    "int_fast32_t",
    "PixelType_Gvsp_YCBCR422_8",
    "PixelType_Gvsp_COORD3D_DEPTH_PLUS_MASK",
    "uint8_t",
    "PixelType_Gvsp_RGB10V1_Packed",
    "uint_least16_t",
    "PixelType_Gvsp_RGB8_Planar",
    "intptr_t",
    "uint_least64_t",
    "int_least32_t",
    "PixelType_Gvsp_RGB16_Packed",
    "PixelType_Gvsp_RGBA8_Packed",
    "PixelType_Gvsp_RGB8_Packed",
    "PixelType_Gvsp_Mono8_Signed",
    "int_least16_t",
    "uint_fast16_t",
    "PixelType_Gvsp_BayerBG10_Packed",
    "PixelType_Gvsp_YCBCR709_422_8_CBYCRY",
    "PixelType_Gvsp_YUV411_Packed",
    "intmax_t",
    "PixelType_Gvsp_BayerBG12_Packed",
    "int16_t",
    "int_fast64_t",
    "PixelType_Gvsp_BayerRG12",
    "PixelType_Gvsp_BayerRG10",
    "PixelType_Gvsp_BayerRG16",
    "uint_fast32_t",
    "PixelType_Gvsp_YCBCR709_411_8_CBYYCRYY",
    "int_least8_t",
    "PixelType_Gvsp_BayerGB12_Packed",
    "PixelType_Gvsp_Coord3D_AC32f",
    "uint_least8_t",
    "PixelType_Gvsp_BayerRG12_Packed",
    "PixelType_Gvsp_Undefined",
    "PixelType_Gvsp_YUV422_Packed",
    "PixelType_Gvsp_RGB10_Packed",
    "PixelType_Gvsp_RGB12_Planar",
    "PixelType_Gvsp_YCBCR709_422_8",
    "uint16_t",
    "uint_fast8_t",
    "PixelType_Gvsp_BGR8_Packed",
    "PixelType_Gvsp_Jpeg",
    "int32_t",
    "PixelType_Gvsp_BayerGR10_Packed",
    "PixelType_Gvsp_BayerBG12",
    "PixelType_Gvsp_BayerBG10",
    "PixelType_Gvsp_BayerBG16",
    "PixelType_Gvsp_Mono12",
    "PixelType_Gvsp_RGB16_Planar",
    "PixelType_Gvsp_Mono4p",
    "PixelType_Gvsp_BayerRG10_Packed",
    "PixelType_Gvsp_Mono8",
    "uint_least32_t",
    "PixelType_Gvsp_BayerGR16",
    "PixelType_Gvsp_BayerGR10",
    "PixelType_Gvsp_BGRA8_Packed",
    "PixelType_Gvsp_BayerGR12",
    "uintptr_t",
    "PixelType_Gvsp_Mono12_Packed",
    "int8_t",
    "PixelType_Gvsp_YCBCR709_8_CBYCR",
    "PixelType_Gvsp_YCBCR601_422_8",
    "PixelType_Gvsp_YCBCR411_8_CBYYCRYY",
    "int_least64_t",
    "PixelType_Gvsp_BGR12_Packed",
    "uint64_t",
    "PixelType_Gvsp_BayerGR12_Packed",
    "PixelType_Gvsp_YCBCR601_411_8_CBYYCRYY",
    "PixelType_Gvsp_RGB10_Planar",
    "PixelType_Gvsp_BayerGB16",
    "PixelType_Gvsp_BayerGB10",
    "uintmax_t",
    "PixelType_Gvsp_BayerGB12",
    "PixelType_Gvsp_BGR565_Packed",
    "int64_t",
    "int_fast16_t",
    "PixelType_Gvsp_Mono1p",
    "PixelType_Gvsp_YUV444_Packed",
    "PixelType_Gvsp_YUV422_YUYV_Packed",
    "PixelType_Gvsp_BayerBG8",
    "int_fast8_t",
    "PixelType_Gvsp_BGR10_Packed",
    "PixelType_Gvsp_BayerGB10_Packed",
    "PixelType_Gvsp_Coord3D_ABC32f_Planar",
    "PixelType_Gvsp_Coord3D_ABC32f",
    "PixelType_Gvsp_YCBCR422_8_CBYCRY",
    "PixelType_Gvsp_RGB12_Packed",
    "PixelType_Gvsp_BayerGR8",
    "MvGvspPixelType",
    "PixelType_Gvsp_Mono10",
    "PixelType_Gvsp_Mono16",
    "PixelType_Gvsp_Mono2p",
    "PixelType_Gvsp_Mono14",
    "PixelType_Gvsp_RGB10V2_Packed",
    "PixelType_Gvsp_RGB12V1_Packed",
    "PixelType_Gvsp_Mono10_Packed",
    "uint_fast64_t",
    "PixelType_Gvsp_YCBCR601_8_CBYCR",
    "PixelType_Gvsp_BayerGB8",
    "PixelType_Gvsp_YCBCR8_CBYCR",
    "PixelType_Gvsp_RGB565_Packed",
    "uint32_t",
    "PixelType_Gvsp_YCBCR601_422_8_CBYCRY",
]



"""CLI entrypoints for hikrobotcamlib"""
import logging

import click
from libadvian.logging import init_logging

from hikrobotcamlib import __version__, DeviceList
from hikrobotcamlib.types import DeviceTransport

LOGGER = logging.getLogger(__name__)


@click.command()
@click.version_option(version=__version__)
@click.option("-l", "--loglevel", help="Python log level, 10=DEBUG, 20=INFO, 30=WARNING, 40=CRITICAL", default=30)
@click.option("-v", "--verbose", count=True, help="Shorthand for info/debug loglevel (-v/-vv)")
@click.option("--usb", is_flag=True, help="Use USB transport")
@click.option("--gige/--no-gige", is_flag=True, help="Use GigE transport", default=True)
def hikrobotcamlib_cli(loglevel: int, verbose: int, usb: bool, gige: bool) -> None:
    """List cameras"""
    if verbose == 1:
        loglevel = 20
    if verbose >= 2:
        loglevel = 10
    init_logging(loglevel)
    LOGGER.setLevel(loglevel)
    transports = 0
    if usb:
        transports |= DeviceTransport.USB
    if gige:
        transports |= DeviceTransport.GIGE

    for dev in DeviceList(transports):
        click.echo("Found {}".format(dev))



"""Wrappers fpr ctypes"""
from typing import Any, Union, Tuple
import logging
import ctypes
from dataclasses import dataclass, field
import enum

from .hiklibs.MvCameraControl_class import MvCamera
from .hiklibs.MvCameraControl_header import MV_FRAME_OUT_INFO_EX, MV_CC_DEVICE_INFO, MV_CC_DEVICE_INFO_LIST
from .hiklibs.CameraParams_const import (
    MV_USB_DEVICE,
    MV_GIGE_DEVICE,
    MV_1394_DEVICE,
    MV_CAMERALINK_DEVICE,
    MV_UNKNOW_DEVICE,
)
from .hiklibs import PixelType_header
from .hiklibs.MvErrorDefine_const import MV_E_RESOURCE
from .errors import EnumerationError, mv_errstr, InvalidTransport, FrameError
from .helpers import get_octets, get_str


LOGGER = logging.getLogger(__name__)

FrameInfoPtrType = ctypes.POINTER(MV_FRAME_OUT_INFO_EX)
FrameDataPtrType = ctypes.POINTER(ctypes.c_ubyte)
FrameCallbackType = ctypes.CFUNCTYPE(None, FrameDataPtrType, FrameInfoPtrType, ctypes.c_void_p)

DeviceInfoPtrType = ctypes.POINTER(MV_CC_DEVICE_INFO)


class DeviceTransport(enum.IntEnum):
    """Device transport type"""

    GIGE = MV_GIGE_DEVICE
    USB = MV_USB_DEVICE
    FW = MV_1394_DEVICE
    LNK = MV_CAMERALINK_DEVICE
    UNKNOWN = MV_UNKNOW_DEVICE


@dataclass
class GigEDeviceInfo:
    """GigE specific info"""

    ptr: Any = field(repr=False)
    addr: str = field(init=False)

    def __post_init__(self) -> None:
        """copy the data"""
        parts = get_octets(self.ptr.nCurrentIp)
        self.addr = ".".join((str(part) for part in parts))


@dataclass
class USBDeviceInfo:
    """USB specific info"""

    ptr: Any = field(repr=False)
    vid: int = field(init=False)
    pid: int = field(init=False)
    devno: int = field(init=False)
    guid: str = field(init=False, default="")

    def __post_init__(self) -> None:
        """copy the data"""
        self.vid = int(self.ptr.idVendor)
        self.pid = int(self.ptr.idProduct)
        self.devno = int(self.ptr.nDeviceNumber)
        self.guid = get_str(self.ptr.chDeviceGUID)


@dataclass
class ModelInfo:
    """Model specific info"""

    ptr: Any = field(repr=False)
    manufacturer: str = field(init=False, default="")
    model: str = field(init=False, default="")
    version: str = field(init=False, default="")

    def __post_init__(self) -> None:
        """copy the data"""
        self.manufacturer = get_str(self.ptr.chManufacturerName)
        self.model = get_str(self.ptr.chModelName)
        self.version = get_str(self.ptr.chDeviceVersion)


ExtraTypes = Union[GigEDeviceInfo, USBDeviceInfo]


@dataclass
class DeviceInfo:  # pylint: disable=R0902
    """Pythonic wrapper for the device info pointer"""

    ptr: Any = field(repr=False)
    # Common fields
    transport: DeviceTransport = field(init=False)
    macaddr: str = field(init=False, default="")
    serialno: str = field(init=False, default="")
    user_name: str = field(init=False, default="")
    model: ModelInfo = field(init=False)

    # Transport specific info
    extra: ExtraTypes = field(init=False)

    # Private pointers etc
    _ptrcontent: Any = field(init=False, repr=False)

    def __post_init__(self) -> None:
        """copy the data"""
        self._ptrcontent = ctypes.cast(self.ptr, DeviceInfoPtrType).contents
        mah = get_octets(self._ptrcontent.nMacAddrHigh)
        mal = get_octets(self._ptrcontent.nMacAddrLow)
        self.macaddr = f"{mah[2]:02X}:{mah[3]:02X}:{mal[0]:02X}:{mal[1]:02X}:{mal[2]:02X}:{mal[3]:02X}"
        self.transport = DeviceTransport(self._ptrcontent.nTLayerType)
        if self.transport == DeviceTransport.GIGE:
            self._init_gige()
        elif self.transport == DeviceTransport.USB:
            self._init_usb()
        else:
            raise InvalidTransport(f"Do not know hot to handle {self.transport}")

    @property
    def mvcamhandle(self) -> Any:
        """Return the cast pointer to be used to create handles"""
        return self._ptrcontent

    def _init_gige(self) -> None:
        """Gige transport init"""
        self._read_common_strings(self._ptrcontent.SpecialInfo.stGigEInfo)
        self.extra = GigEDeviceInfo(self._ptrcontent.SpecialInfo.stGigEInfo)

    def _read_common_strings(self, ptr: Any) -> None:
        """Read properties that have same names in all transports"""
        self.model = ModelInfo(ptr)
        self.serialno = get_str(ptr.chSerialNumber)
        self.user_name = get_str(ptr.chUserDefinedName)

    def _init_usb(self) -> None:
        """USB transport init"""
        self._read_common_strings(self._ptrcontent.SpecialInfo.stUsb3VInfo)
        self.extra = USBDeviceInfo(self._ptrcontent.SpecialInfo.stUsb3VInfo)


@dataclass
class DeviceList:
    """Pythonic wrapper for the device list"""

    transports: int = field(default=DeviceTransport.GIGE)

    _listptr: Any = field(init=False, repr=False)
    _iteridx: int = field(default=0, init=False)

    def __post_init__(self) -> None:
        """Init pointers list things"""
        self._listptr = MV_CC_DEVICE_INFO_LIST()
        ret = MvCamera.MV_CC_EnumDevices(self.transports, self._listptr)  # type: ignore[no-untyped-call]
        if ret != 0:
            if ret == MV_E_RESOURCE:
                LOGGER.warning("Resource error, try adjusting your transport choices")
            raise EnumerationError(f"Error {mv_errstr(ret)}")

    def __iter__(self) -> "DeviceList":
        """We implement iterator protocol"""
        self._iteridx = 0
        return self

    def __getitem__(self, idx: int) -> DeviceInfo:
        """get specific device"""
        idx = int(idx)  # make sure it is an int
        if idx < 0:
            raise IndexError
        if idx > (self._listptr.nDeviceNum - 1):
            raise IndexError
        return DeviceInfo(self._listptr.pDeviceInfo[idx])

    def __len__(self) -> int:
        """number of devices"""
        return int(self._listptr.nDeviceNum)

    def __next__(self) -> DeviceInfo:
        """Return next device"""
        if self._iteridx > (self._listptr.nDeviceNum - 1):
            raise StopIteration
        device = DeviceInfo(self._listptr.pDeviceInfo[self._iteridx])
        self._iteridx += 1
        return device


MV_PIXEL_TYPES = {
    getattr(PixelType_header, name): name.replace("PixelType_Gvsp_", "")
    for name in dir(PixelType_header)
    if name.startswith("PixelType_Gvsp_")
}


@dataclass
class Frame:  # pylint: disable=R0902
    """Wrap frame pointers to nicer accessor"""

    infoptr: Any = field(repr=False)
    dataptr: Any = field(repr=False)

    frameno: int = field(init=False)
    timestamp: int = field(init=False)  # device ticks
    size: Tuple[int, int] = field(init=False, default=(0, 0))  # w,h
    type: str = field(init=False)
    exposure: float = field(init=False)
    len: int = field(init=False)

    # Private pointers etc
    _inf: Any = field(init=False, repr=False)

    def __post_init__(self) -> None:
        """Copy data"""
        self._inf = ctypes.cast(self.infoptr, FrameInfoPtrType).contents
        if not self._inf:
            raise FrameError("Invalid info pointer")
        self.timestamp = (self._inf.nDevTimeStampHigh << ctypes.sizeof(ctypes.c_uint)) + self._inf.nDevTimeStampLow
        self.size = (int(self._inf.nWidth), int(self._inf.nHeight))
        self.frameno = int(self._inf.nFrameNum)
        self.type = MV_PIXEL_TYPES[self._inf.enPixelType]
        self.exposure = float(self._inf.fExposureTime)  # seems to be 0.0 for some reason
        self.len = int(self._inf.nFrameLen)

    @property
    def data(self) -> bytes:
        """Return the raw bytes"""
        buffer_pntr = ctypes.cast(self.dataptr, ctypes.POINTER(ctypes.c_ubyte * self.len))
        return bytes(buffer_pntr.contents)



# -- coding: utf-8 --
# pylint: skip-file
import sys
import copy
import ctypes
import os
import platform
from pathlib import Path
import logging

from ctypes import *

from .CameraParams_const import *
from .CameraParams_header import *
from .MvCameraControl_header import *
from .MvErrorDefine_const import *
from .PixelType_const import *
from .PixelType_header import *


LOGGER = logging.getLogger(__name__)
LOGGER.debug("platform.system()={}".format(platform.system()))

libpath = None
if platform.system() == "Darwin":
    libpath = Path("/Library/MVS_SDK/lib") / "libMvCameraControl.dylib"
if platform.system() == "Linux":
    archpath = platform.machine()
    if archpath == "x86_64":
        archpath = "64"
    libpath = Path("/opt/MvCamCtrlSDK/lib") / archpath / "libMvCameraControl.so"

LOGGER.debug("libpath={} exists={}".format(libpath, libpath.exists()))
if not libpath or not libpath.exists():
    raise RuntimeError("Cannot determine cdll to load")

MvCamCtrldll = ctypes.cdll.LoadLibrary(str(libpath.resolve()))


# 用于回调函数传入相机实例
class _MV_PY_OBJECT_(Structure):
    pass


_MV_PY_OBJECT_._fields_ = [
    ("PyObject", py_object),
]
MV_PY_OBJECT = _MV_PY_OBJECT_


class MvCamera:
    def __init__(self):
        self._handle = c_void_p()  # 记录当前连接设备的句柄
        self.handle = pointer(self._handle)  # 创建句柄指针

    @staticmethod
    def MV_CC_GetSDKVersion():
        MvCamCtrldll.MV_CC_GetSDKVersion.restype = c_uint
        # C原型:unsigned int MV_CC_GetSDKVersion()
        return MvCamCtrldll.MV_CC_GetSDKVersion()

    # 枚举设备
    @staticmethod
    def MV_CC_EnumDevices(nTLayerType, stDevList):
        MvCamCtrldll.MV_CC_EnumDevices.argtype = (c_uint, c_void_p)
        MvCamCtrldll.MV_CC_EnumDevices.restype = c_uint
        # C原型:int MV_CC_EnumDevices(unsigned int nTLayerType, MV_CC_DEVICE_INFO_LIST* pstDevList)
        return MvCamCtrldll.MV_CC_EnumDevices(c_uint(nTLayerType), byref(stDevList))

    # 创建句柄
    def MV_CC_CreateHandle(self, stDevInfo):
        MvCamCtrldll.MV_CC_DestroyHandle.argtype = c_void_p
        MvCamCtrldll.MV_CC_DestroyHandle.restype = c_uint
        MvCamCtrldll.MV_CC_DestroyHandle(self.handle)

        MvCamCtrldll.MV_CC_CreateHandle.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_CreateHandle.restype = c_uint
        # C原型:int MV_CC_CreateHandle(void ** handle, MV_CC_DEVICE_INFO* pstDevInfo)
        return MvCamCtrldll.MV_CC_CreateHandle(byref(self.handle), byref(stDevInfo))

    # 创建句柄（不生成日志）
    def MV_CC_CreateHandleWithoutLog(self, stDevInfo):
        MvCamCtrldll.MV_CC_DestroyHandle.argtype = c_void_p
        MvCamCtrldll.MV_CC_DestroyHandle.restype = c_uint
        MvCamCtrldll.MV_CC_DestroyHandle(self.handle)

        MvCamCtrldll.MV_CC_CreateHandleWithoutLog.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_CreateHandleWithoutLog.restype = c_uint
        # C原型:int MV_CC_CreateHandleWithoutLog(void ** handle, MV_CC_DEVICE_INFO* pstDevInfo)
        return MvCamCtrldll.MV_CC_CreateHandleWithoutLog(byref(self.handle), byref(stDevInfo))

    # 销毁句柄
    def MV_CC_DestroyHandle(self):
        MvCamCtrldll.MV_CC_DestroyHandle.argtype = c_void_p
        MvCamCtrldll.MV_CC_DestroyHandle.restype = c_uint
        return MvCamCtrldll.MV_CC_DestroyHandle(self.handle)

    # 打开设备
    def MV_CC_OpenDevice(self, nAccessMode=MV_ACCESS_Exclusive, nSwitchoverKey=0):
        MvCamCtrldll.MV_CC_OpenDevice.argtype = (c_void_p, c_uint32, c_uint16)
        MvCamCtrldll.MV_CC_OpenDevice.restype = c_uint
        # C原型:int MV_CC_OpenDevice(void* handle, unsigned int nAccessMode, unsigned short nSwitchoverKey)
        return MvCamCtrldll.MV_CC_OpenDevice(self.handle, nAccessMode, nSwitchoverKey)

    # 关闭设备
    def MV_CC_CloseDevice(self):
        MvCamCtrldll.MV_CC_CloseDevice.argtype = c_void_p
        MvCamCtrldll.MV_CC_CloseDevice.restype = c_uint
        return MvCamCtrldll.MV_CC_CloseDevice(self.handle)

    # 注册取流回调
    def MV_CC_RegisterImageCallBackEx(self, CallBackFun, pUser):
        MvCamCtrldll.MV_CC_RegisterImageCallBackEx.argtype = (c_void_p, c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_RegisterImageCallBackEx.restype = c_uint
        # C原型:int MV_CC_RegisterImageCallBackEx(void* handle, void(* cbOutput)(unsigned char * pData, MV_FRAME_OUT_INFO_EX* pFrameInfo, void* pUser),void* pUser);
        return MvCamCtrldll.MV_CC_RegisterImageCallBackEx(self.handle, CallBackFun, pUser)

    # 开始取流
    def MV_CC_StartGrabbing(self):
        MvCamCtrldll.MV_CC_StartGrabbing.argtype = c_void_p
        MvCamCtrldll.MV_CC_StartGrabbing.restype = c_uint
        return MvCamCtrldll.MV_CC_StartGrabbing(self.handle)

    # 停止取流
    def MV_CC_StopGrabbing(self):
        MvCamCtrldll.MV_CC_StopGrabbing.argtype = c_void_p
        MvCamCtrldll.MV_CC_StopGrabbing.restype = c_uint
        return MvCamCtrldll.MV_CC_StopGrabbing(self.handle)

    # 主动方式取流
    def MV_CC_GetOneFrameTimeout(self, pData, nDataSize, stFrameInfo, nMsec=1000):
        MvCamCtrldll.MV_CC_GetOneFrameTimeout.argtype = (c_void_p, c_void_p, c_uint, c_void_p, c_uint)
        MvCamCtrldll.MV_CC_GetOneFrameTimeout.restype = c_uint
        # C原型:int MV_CC_GetOneFrameTimeout(void* handle, unsigned char * pData , unsigned int nDataSize, MV_FRAME_OUT_INFO_EX* pFrameInfo, unsigned int nMsec)
        return MvCamCtrldll.MV_CC_GetOneFrameTimeout(self.handle, pData, nDataSize, byref(stFrameInfo), nMsec)

    # 设置SDK内部图像缓存节点个数，范围[1, 30]，在抓图前调用
    def MV_CC_SetImageNodeNum(self, nNum):
        MvCamCtrldll.MV_CC_SetImageNodeNum.argtype = (c_void_p, c_uint)
        MvCamCtrldll.MV_CC_SetImageNodeNum.restype = c_uint
        # C原型:int MV_CC_SetImageNodeNum(void* handle, unsigned int nNum)
        return MvCamCtrldll.MV_CC_SetImageNodeNum(self.handle, nNum)

    # 获取Integer型属性值
    def MV_CC_GetIntValue(self, strKey, stIntValue):
        MvCamCtrldll.MV_CC_GetIntValue.argtype = (c_void_p, c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_GetIntValue.restype = c_uint
        # C原型:int MV_CC_GetIntValue(void* handle,char* strKey,MVCC_INTVALUE *pIntValue)
        return MvCamCtrldll.MV_CC_GetIntValue(self.handle, strKey.encode("ascii"), byref(stIntValue))

    # 设置Integer型属性值
    def MV_CC_SetIntValue(self, strKey, nValue):
        MvCamCtrldll.MV_CC_SetIntValue.argtype = (c_void_p, c_void_p, c_uint32)
        MvCamCtrldll.MV_CC_SetIntValue.restype = c_uint
        # C原型:int MV_CC_SetIntValue(void* handle,char* strKey,unsigned int nValue)
        return MvCamCtrldll.MV_CC_SetIntValue(self.handle, strKey.encode("ascii"), c_uint32(nValue))

    # 获取Enum型属性值
    def MV_CC_GetEnumValue(self, strKey, stEnumValue):
        MvCamCtrldll.MV_CC_GetEnumValue.argtype = (c_void_p, c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_GetEnumValue.restype = c_uint
        # C原型:int MV_CC_GetEnumValue(void* handle,char* strKey,MVCC_ENUMVALUE *pEnumValue)
        return MvCamCtrldll.MV_CC_GetEnumValue(self.handle, strKey.encode("ascii"), byref(stEnumValue))

    # 设置Enum型属性值
    def MV_CC_SetEnumValue(self, strKey, nValue):
        MvCamCtrldll.MV_CC_SetEnumValue.argtype = (c_void_p, c_void_p, c_uint32)
        MvCamCtrldll.MV_CC_SetEnumValue.restype = c_uint
        # C原型:int MV_CC_SetEnumValue(void* handle,char* strKey,unsigned int nValue)
        return MvCamCtrldll.MV_CC_SetEnumValue(self.handle, strKey.encode("ascii"), c_uint32(nValue))

    # 设置Enum型属性值
    def MV_CC_SetEnumValueByString(self, strKey, sValue):
        MvCamCtrldll.MV_CC_SetEnumValueByString.argtype = (c_void_p, c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_SetEnumValueByString.restype = c_uint
        # C原型:int MV_CC_SetEnumValueByString(void* handle,char* strKey,char* sValue)
        return MvCamCtrldll.MV_CC_SetEnumValueByString(self.handle, strKey.encode("ascii"), sValue.encode("ascii"))

    # 获取Float型属性值
    def MV_CC_GetFloatValue(self, strKey, stFloatValue):
        MvCamCtrldll.MV_CC_GetFloatValue.argtype = (c_void_p, c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_GetFloatValue.restype = c_uint
        # C原型:int MV_CC_GetFloatValue(void* handle,char* strKey,MVCC_FLOATVALUE *pFloatValue)
        return MvCamCtrldll.MV_CC_GetFloatValue(self.handle, strKey.encode("ascii"), byref(stFloatValue))

    # 设置Float型属性值
    def MV_CC_SetFloatValue(self, strKey, fValue):
        MvCamCtrldll.MV_CC_SetFloatValue.argtype = (c_void_p, c_void_p, c_float)
        MvCamCtrldll.MV_CC_SetFloatValue.restype = c_uint
        # C原型:int MV_CC_SetFloatValue(void* handle,char* strKey,float fValue)
        return MvCamCtrldll.MV_CC_SetFloatValue(self.handle, strKey.encode("ascii"), c_float(fValue))

    # 获取Boolean型属性值
    def MV_CC_GetBoolValue(self, strKey, BoolValue):
        MvCamCtrldll.MV_CC_GetBoolValue.argtype = (c_void_p, c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_GetBoolValue.restype = c_uint
        # C原型:int MV_CC_GetBoolValue(void* handle,char* strKey,bool *pBoolValue)
        return MvCamCtrldll.MV_CC_GetBoolValue(self.handle, strKey.encode("ascii"), byref(BoolValue))

    # 设置Boolean型属性值
    def MV_CC_SetBoolValue(self, strKey, bValue):
        MvCamCtrldll.MV_CC_SetBoolValue.argtype = (c_void_p, c_void_p, c_bool)
        MvCamCtrldll.MV_CC_SetBoolValue.restype = c_uint
        # C原型:int MV_CC_SetBoolValue(void* handle,char* strKey,bool bValue)
        return MvCamCtrldll.MV_CC_SetBoolValue(self.handle, strKey.encode("ascii"), bValue)

    # 获取String型属性值
    def MV_CC_GetStringValue(self, strKey, StringValue):
        MvCamCtrldll.MV_CC_GetStringValue.argtype = (c_void_p, c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_GetStringValue.restype = c_uint
        # C原型:int MV_CC_GetStringValue(void* handle,char* strKey,MVCC_STRINGVALUE *pStringValue)
        return MvCamCtrldll.MV_CC_GetStringValue(self.handle, strKey.encode("ascii"), byref(StringValue))

    # 设置String型属性值
    def MV_CC_SetStringValue(self, strKey, sValue):
        MvCamCtrldll.MV_CC_SetStringValue.argtype = (c_void_p, c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_SetStringValue.restype = c_uint
        # C原型:int MV_CC_SetStringValue(void* handle,char* strKey,char * sValue)
        return MvCamCtrldll.MV_CC_SetStringValue(self.handle, strKey.encode("ascii"), sValue.encode("ascii"))

    # 设置Command型属性值
    def MV_CC_SetCommandValue(self, strKey):
        MvCamCtrldll.MV_CC_SetCommandValue.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_SetCommandValue.restype = c_uint
        # C原型:int MV_CC_SetCommandValue(void* handle,char* strKey)
        return MvCamCtrldll.MV_CC_SetCommandValue(self.handle, strKey.encode("ascii"))

    # 注册异常消息回调
    def MV_CC_RegisterExceptionCallBack(self, ExceptionCallBackFun, pUser):
        MvCamCtrldll.MV_CC_RegisterExceptionCallBack.argtype = (c_void_p, c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_RegisterExceptionCallBack.restype = c_uint
        # C原型:int MV_CC_RegisterExceptionCallBack(void* handle, void(* cbException)(unsigned int nMsgType, void* pUser),void* pUser)
        return MvCamCtrldll.MV_CC_RegisterExceptionCallBack(self.handle, ExceptionCallBackFun, pUser)

    # 注册事件回调
    def MV_CC_RegisterEventCallBackEx(self, pEventName, EventCallBackFun, pUser):
        MvCamCtrldll.MV_CC_RegisterEventCallBackEx.argtype = (c_void_p, c_void_p, c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_RegisterEventCallBackEx.restype = c_uint
        # C原型:int MV_CC_RegisterEventCallBackEx(void* handle, char* pEventName,void(* cbEvent)(MV_EVENT_OUT_INFO * pEventInfo, void* pUser),void* pUser)
        return MvCamCtrldll.MV_CC_RegisterEventCallBackEx(
            self.handle, pEventName.encode("ascii"), EventCallBackFun, pUser
        )

    # 强制IP
    def MV_GIGE_ForceIpEx(self, nIP, nSubNetMask, nDefaultGateWay):
        MvCamCtrldll.MV_GIGE_ForceIpEx.argtype = (c_void_p, c_uint, c_uint, c_uint)
        MvCamCtrldll.MV_GIGE_ForceIpEx.restype = c_uint
        # C原型:int MV_GIGE_ForceIpEx(void* handle, unsigned int nIP, unsigned int nSubNetMask, unsigned int nDefaultGateWay)
        return MvCamCtrldll.MV_GIGE_ForceIpEx(self.handle, c_uint(nIP), c_uint(nSubNetMask), c_uint(nDefaultGateWay))

    # 配置IP方式
    def MV_GIGE_SetIpConfig(self, nType):
        MvCamCtrldll.MV_GIGE_SetIpConfig.argtype = (c_void_p, c_uint)
        MvCamCtrldll.MV_GIGE_SetIpConfig.restype = c_uint
        # C原型:int MV_GIGE_SetIpConfig(void* handle, unsigned int nType)
        return MvCamCtrldll.MV_GIGE_SetIpConfig(self.handle, c_uint(nType))

    # 设置传输模式，可以为单播模式、组播模式等
    def MV_GIGE_SetTransmissionType(self, stTransmissionType):
        MvCamCtrldll.MV_GIGE_SetTransmissionType.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_GIGE_SetTransmissionType.restype = c_uint
        # C原型:int MV_GIGE_SetTransmissionType(void* handle, MV_TRANSMISSION_TYPE * pstTransmissionType)
        return MvCamCtrldll.MV_GIGE_SetTransmissionType(self.handle, byref(stTransmissionType))

    # 保存一张图片
    def MV_CC_SaveImageEx2(self, stSaveParam):
        MvCamCtrldll.MV_CC_SaveImageEx2.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_SaveImageEx2.restype = c_uint
        # C原型:int MV_CC_SaveImageEx2(void* handle, MV_SAVE_IMAGE_PARAM_EX* pSaveParam)
        return MvCamCtrldll.MV_CC_SaveImageEx2(self.handle, byref(stSaveParam))

    # 像素格式转换
    def MV_CC_ConvertPixelType(self, stConvertParam):
        MvCamCtrldll.MV_CC_ConvertPixelType.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_ConvertPixelType.restype = c_uint
        # C原型:int MV_CC_ConvertPixelType(void* handle, MV_CC_PIXEL_CONVERT_PARAM* pstCvtParam)
        return MvCamCtrldll.MV_CC_ConvertPixelType(self.handle, byref(stConvertParam))

    # 保存属性节点
    def MV_CC_FeatureSave(self, pFileName):
        MvCamCtrldll.MV_CC_FeatureSave.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_FeatureSave.restype = c_uint
        # C原型:int MV_CC_FeatureSave(void* handle, char* pFileName)
        return MvCamCtrldll.MV_CC_FeatureSave(self.handle, pFileName.encode("ascii"))

    # 加载属性节点
    def MV_CC_FeatureLoad(self, pFileName):
        MvCamCtrldll.MV_CC_FeatureLoad.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_FeatureLoad.restype = c_uint
        # C原型:int MV_CC_FeatureLoad(void* handle, char* pFileName)
        return MvCamCtrldll.MV_CC_FeatureLoad(self.handle, pFileName.encode("ascii"))

    # fileaccess read
    def MV_CC_FileAccessRead(self, stFileAccess):
        MvCamCtrldll.MV_CC_FileAccessRead.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_FileAccessRead.restype = c_uint
        # C原型:int MV_CC_FileAccessRead(void* handle, MV_CC_FILE_ACCESS * pstFileAccess)
        return MvCamCtrldll.MV_CC_FileAccessRead(self.handle, byref(stFileAccess))

    # fileaccess write
    def MV_CC_FileAccessWrite(self, stFileAccess):
        MvCamCtrldll.MV_CC_FileAccessWrite.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_FileAccessWrite.restype = c_uint
        # C原型:int MV_CC_FileAccessWrite(void* handle, MV_CC_FILE_ACCESS * pstFileAccess)
        return MvCamCtrldll.MV_CC_FileAccessWrite(self.handle, byref(stFileAccess))

    # 获取文件存取进度
    def MV_CC_GetFileAccessProgress(self, stFileAccessProgress):
        MvCamCtrldll.MV_CC_GetFileAccessProgress.argtype = (c_void_p, c_void_p)
        MvCamCtrldll.MV_CC_GetFileAccessProgress.restype = c_uint
        # C原型:int MV_CC_GetFileAccessProgress(void* handle, MV_CC_FILE_ACCESS_PROGRESS * pstFileAccessProgress)
        return MvCamCtrldll.MV_CC_GetFileAccessProgress(self.handle, byref(stFileAccessProgress))

    # 获取网络最佳包大小
    def MV_CC_GetOptimalPacketSize(self):
        MvCamCtrldll.MV_CC_GetOptimalPacketSize.argtype = c_void_p
        MvCamCtrldll.MV_CC_GetOptimalPacketSize.restype = c_uint
        # C原型:int __stdcall MV_CC_GetOptimalPacketSize(void* handle);
        return MvCamCtrldll.MV_CC_GetOptimalPacketSize(self.handle)



# generated by 'xml2py'
# flags '-c -d -v C:\test_h\CameraParams.xml -o CameraParams_header.py'
# pylint: skip-file
from ctypes import *

STRING = c_char_p


MV_EXPOSURE_MODE_TRIGGER_WIDTH = 1
MV_TRIGGER_SOURCE_FrequencyConverter = 8
MV_TRIGGER_SOURCE_SOFTWARE = 7
MV_TRIGGER_SOURCE_COUNTER0 = 4
MV_BALANCEWHITE_AUTO_OFF = 0
MV_TRIGGER_SOURCE_LINE3 = 3
MV_TRIGGER_SOURCE_LINE2 = 2
MV_TRIGGER_SOURCE_LINE1 = 1
MV_EXPOSURE_MODE_TIMED = 0
MV_TRIGGER_SOURCE_LINE0 = 0
AM_NA = 1
PixelType_Gvsp_BayerGR8 = 17301512
PixelType_Gvsp_Mono16 = 17825799
PixelType_Gvsp_Mono14 = 17825829
PixelType_Gvsp_Mono12 = 17825797
PixelType_Gvsp_Mono10 = 17825795
V_Undefined = 99
PixelType_Gvsp_Mono12_Packed = 17563654
V_Invisible = 3
PixelType_Gvsp_Mono8 = 17301505
V_Guru = 2
PixelType_Gvsp_Mono4p = 17039417
V_Expert = 1
PixelType_Gvsp_BayerGR16 = 17825838
PixelType_Gvsp_Mono1p = 16842807
PixelType_Gvsp_Undefined = -1
MV_GAIN_MODE_CONTINUOUS = 2
IFT_IString = 6
PixelType_Gvsp_BayerBG10 = 17825807
MV_ACQ_MODE_CONTINUOUS = 2
MV_EXPOSURE_AUTO_MODE_ONCE = 1
MV_Image_Png = 3
MV_Image_Jpeg = 2
MV_Image_Bmp = 1
MV_ACQ_MODE_MUTLI = 1
MV_EXPOSURE_AUTO_MODE_OFF = 0
PixelType_Gvsp_Jpeg = -2145910783
PixelType_Gvsp_RGB16_Planar = 36700196
IFT_IPort = 11
IFT_IEnumEntry = 10
IFT_ICategory = 8
IFT_IRegister = 7
IFT_IFloat = 5
IFT_IEnumeration = 9
IFT_ICommand = 4
IFT_IBoolean = 3
IFT_IInteger = 2
IFT_IBase = 1
PixelType_Gvsp_COORD3D_DEPTH_PLUS_MASK = -2112094207
IFT_IValue = 0
PixelType_Gvsp_Coord3D_AC32f = 36176066
PixelType_Gvsp_Coord3D_ABC32f_Planar = 39846081
PixelType_Gvsp_Coord3D_ABC32f = 39846080
PixelType_Gvsp_RGB12_Planar = 36700195
PixelType_Gvsp_RGB10_Planar = 36700194
PixelType_Gvsp_YCBCR709_411_8_CBYYCRYY = 34340930
MV_TRIGGER_MODE_OFF = 0
PixelType_Gvsp_YCBCR709_422_8_CBYCRY = 34603077
PixelType_Gvsp_YCBCR709_8_CBYCR = 35127360
PixelType_Gvsp_YCBCR709_422_8 = 34603073
PixelType_Gvsp_YCBCR601_411_8_CBYYCRYY = 34340927
PixelType_Gvsp_YCBCR601_422_8 = 34603070
PixelType_Gvsp_YCBCR601_8_CBYCR = 35127357
MV_TRIGGER_MODE_ON = 1
PixelType_Gvsp_YCBCR411_8_CBYYCRYY = 34340924
PixelType_Gvsp_YCBCR422_8_CBYCRY = 34603075
PixelType_Gvsp_YCBCR422_8 = 34603067
PixelType_Gvsp_YCBCR8_CBYCR = 35127354
PixelType_Gvsp_YUV444_Packed = 35127328
PixelType_Gvsp_YUV422_YUYV_Packed = 34603058
PixelType_Gvsp_YUV422_Packed = 34603039
PixelType_Gvsp_YUV411_Packed = 34340894
PixelType_Gvsp_BGR565_Packed = 34603062
PixelType_Gvsp_RGB565_Packed = 34603061
PixelType_Gvsp_RGB12V1_Packed = 35913780
PixelType_Gvsp_RGB10V2_Packed = 35651613
PixelType_Gvsp_RGB16_Packed = 36700211
PixelType_Gvsp_YCBCR601_422_8_CBYCRY = 34603076
PixelType_Gvsp_RGB12_Packed = 36700186
PixelType_Gvsp_BGR10_Packed = 36700185
MV_GIGE_TRANSTYPE_UNICAST = 0
PixelType_Gvsp_RGB10_Packed = 36700184
PixelType_Gvsp_BGRA8_Packed = 35651607
PixelType_Gvsp_RGBA8_Packed = 35651606
PixelType_Gvsp_BGR8_Packed = 35127317
PixelType_Gvsp_RGB8_Packed = 35127316
PixelType_Gvsp_BayerBG16 = 17825841
PixelType_Gvsp_BayerGB16 = 17825840
PixelType_Gvsp_BayerRG16 = 17825839
PixelType_Gvsp_BayerBG12_Packed = 17563693
PixelType_Gvsp_BayerGB12_Packed = 17563692
PixelType_Gvsp_BayerRG12_Packed = 17563691
PixelType_Gvsp_BayerGR12_Packed = 17563690
PixelType_Gvsp_BayerBG10_Packed = 17563689
PixelType_Gvsp_BayerGB10_Packed = 17563688
MV_GAIN_MODE_ONCE = 1
PixelType_Gvsp_BayerRG10_Packed = 17563687
PixelType_Gvsp_BayerGR10_Packed = 17563686
PixelType_Gvsp_BayerBG12 = 17825811
PixelType_Gvsp_BayerGB12 = 17825810
PixelType_Gvsp_BayerRG12 = 17825809
PixelType_Gvsp_BayerGR12 = 17825808
PixelType_Gvsp_BayerGB10 = 17825806
PixelType_Gvsp_BayerRG10 = 17825805
PixelType_Gvsp_BayerGR10 = 17825804
PixelType_Gvsp_BayerBG8 = 17301515
PixelType_Gvsp_BayerGB8 = 17301514
PixelType_Gvsp_BayerRG8 = 17301513
MV_EXPOSURE_AUTO_MODE_CONTINUOUS = 2
MV_Image_Tif = 4
AM_RW = 4
MV_BALANCEWHITE_AUTO_CONTINUOUS = 1
MV_BALANCEWHITE_AUTO_ONCE = 2
PixelType_Gvsp_Mono2p = 16908344
MV_GAMMA_SELECTOR_SRGB = 2
MV_GAIN_MODE_OFF = 0
PixelType_Gvsp_Mono10_Packed = 17563652
MV_GAMMA_SELECTOR_USER = 1
PixelType_Gvsp_RGB8_Planar = 35127329
PixelType_Gvsp_Mono8_Signed = 17301506
MV_ACQ_MODE_SINGLE = 0
AM_CycleDetect = 6
MV_GIGE_TRANSTYPE_MULTICAST_WITHOUT_RECV = 65537
AM_Undefined = 5
MV_GIGE_TRANSTYPE_UNICAST_WITHOUT_RECV = 65536
MV_GIGE_TRANSTYPE_UNICAST_DEFINED_PORT = 5
AM_RO = 3
MV_GIGE_TRANSTYPE_SUBNETBROADCAST = 3
PixelType_Gvsp_RGB10V1_Packed = 35651612
MV_GIGE_TRANSTYPE_LIMITEDBROADCAST = 2
MV_GIGE_TRANSTYPE_MULTICAST = 1
MV_GIGE_TRANSTYPE_CAMERADEFINED = 4
AM_NI = 0
PixelType_Gvsp_BGR12_Packed = 36700187
MV_Image_Undefined = 0
AM_WO = 2
V_Beginner = 0
int8_t = c_int8
int16_t = c_int16
int32_t = c_int32
int64_t = c_int64
uint8_t = c_uint8
uint16_t = c_uint16
uint32_t = c_uint32
uint64_t = c_uint64
int_least8_t = c_byte
int_least16_t = c_short
int_least32_t = c_int
int_least64_t = c_long
uint_least8_t = c_ubyte
uint_least16_t = c_ushort
uint_least32_t = c_uint
uint_least64_t = c_ulong
int_fast8_t = c_byte
int_fast16_t = c_long
int_fast32_t = c_long
int_fast64_t = c_long
uint_fast8_t = c_ubyte
uint_fast16_t = c_ulong
uint_fast32_t = c_ulong
uint_fast64_t = c_ulong
intptr_t = c_long
uintptr_t = c_ulong
intmax_t = c_long
uintmax_t = c_ulong


# CameraParams.h 21
class _MV_GIGE_DEVICE_INFO_(Structure):
    pass


_MV_GIGE_DEVICE_INFO_._fields_ = [
    # CameraParams.h 21
    ("nIpCfgOption", c_uint),
    ("nIpCfgCurrent", c_uint),
    ("nCurrentIp", c_uint),
    ("nCurrentSubNetMask", c_uint),
    ("nDefultGateWay", c_uint),
    ("chManufacturerName", c_ubyte * 32),
    ("chModelName", c_ubyte * 32),
    ("chDeviceVersion", c_ubyte * 32),
    ("chManufacturerSpecificInfo", c_ubyte * 48),
    ("chSerialNumber", c_ubyte * 16),
    ("chUserDefinedName", c_ubyte * 16),
    ("nNetExport", c_uint),
    ("nReserved", c_uint * 4),
]
MV_GIGE_DEVICE_INFO = _MV_GIGE_DEVICE_INFO_


# CameraParams.h 42
class _MV_USB3_DEVICE_INFO_(Structure):
    pass


_MV_USB3_DEVICE_INFO_._fields_ = [
    # CameraParams.h 42
    ("CrtlInEndPoint", c_ubyte),
    ("CrtlOutEndPoint", c_ubyte),
    ("StreamEndPoint", c_ubyte),
    ("EventEndPoint", c_ubyte),
    ("idVendor", c_ushort),
    ("idProduct", c_ushort),
    ("nDeviceNumber", c_uint),
    ("chDeviceGUID", c_ubyte * 64),
    ("chVendorName", c_ubyte * 64),
    ("chModelName", c_ubyte * 64),
    ("chFamilyName", c_ubyte * 64),
    ("chDeviceVersion", c_ubyte * 64),
    ("chManufacturerName", c_ubyte * 64),
    ("chSerialNumber", c_ubyte * 64),
    ("chUserDefinedName", c_ubyte * 64),
    ("nbcdUSB", c_uint),
    ("nReserved", c_uint * 3),
]
MV_USB3_DEVICE_INFO = _MV_USB3_DEVICE_INFO_


# CameraParams.h 64
class _MV_CC_DEVICE_INFO_(Structure):
    pass


# CameraParams.h 76
class N19_MV_CC_DEVICE_INFO_3DOT_0E(Union):
    pass


N19_MV_CC_DEVICE_INFO_3DOT_0E._fields_ = [
    # CameraParams.h 76
    ("stGigEInfo", MV_GIGE_DEVICE_INFO),
    ("stUsb3VInfo", MV_USB3_DEVICE_INFO),
]
_MV_CC_DEVICE_INFO_._fields_ = [
    # CameraParams.h 64
    ("nMajorVer", c_ushort),
    ("nMinorVer", c_ushort),
    ("nMacAddrHigh", c_uint),
    ("nMacAddrLow", c_uint),
    ("nTLayerType", c_uint),
    ("nReserved", c_uint * 4),
    ("SpecialInfo", N19_MV_CC_DEVICE_INFO_3DOT_0E),
]
MV_CC_DEVICE_INFO = _MV_CC_DEVICE_INFO_


# CameraParams.h 86
class _MV_NETTRANS_INFO_(Structure):
    pass


_MV_NETTRANS_INFO_._fields_ = [
    # CameraParams.h 86
    ("nReviceDataSize", int64_t),
    ("nThrowFrameCount", c_int),
    ("nReserved", c_uint * 5),
]
MV_NETTRANS_INFO = _MV_NETTRANS_INFO_


# CameraParams.h 101
class _MV_CC_DEVICE_INFO_LIST_(Structure):
    pass


_MV_CC_DEVICE_INFO_LIST_._fields_ = [
    # CameraParams.h 101
    ("nDeviceNum", c_uint),
    ("pDeviceInfo", POINTER(MV_CC_DEVICE_INFO) * 256),
]
MV_CC_DEVICE_INFO_LIST = _MV_CC_DEVICE_INFO_LIST_


# CameraParams.h 110
class _MV_FRAME_OUT_INFO_(Structure):
    pass


# values for enumeration 'MvGvspPixelType'
MvGvspPixelType = int64_t  # enum
_MV_FRAME_OUT_INFO_._fields_ = [
    # CameraParams.h 110
    ("nWidth", c_ushort),
    ("nHeight", c_ushort),
    ("enPixelType", MvGvspPixelType),
    ("nFrameNum", c_uint),
    ("nDevTimeStampHigh", c_uint),
    ("nDevTimeStampLow", c_uint),
    ("nReserved0", c_uint),
    ("nHostTimeStamp", int64_t),
    ("nFrameLen", c_uint),
    ("nLostPacket", c_uint),
    ("nReserved", c_uint * 2),
]
MV_FRAME_OUT_INFO = _MV_FRAME_OUT_INFO_


# CameraParams.h 129
class _MV_FRAME_OUT_INFO_EX_(Structure):
    pass


_MV_FRAME_OUT_INFO_EX_._fields_ = [
    # CameraParams.h 129
    ("nWidth", c_ushort),
    ("nHeight", c_ushort),
    ("enPixelType", MvGvspPixelType),
    ("nFrameNum", c_uint),
    ("nDevTimeStampHigh", c_uint),
    ("nDevTimeStampLow", c_uint),
    ("nReserved0", c_uint),
    ("nHostTimeStamp", int64_t),
    ("nFrameLen", c_uint),
    ("nSecondCount", c_uint),
    ("nCycleCount", c_uint),
    ("nCycleOffset", c_uint),
    ("fGain", c_float),
    ("fExposureTime", c_float),
    ("nAverageBrightness", c_uint),
    ("nRed", c_uint),
    ("nGreen", c_uint),
    ("nBlue", c_uint),
    ("nFrameCounter", c_uint),
    ("nTriggerIndex", c_uint),
    ("nInput", c_uint),
    ("nOutput", c_uint),
    ("nOffsetX", c_ushort),
    ("nOffsetY", c_ushort),
    ("nChunkWidth", c_ushort),
    ("nChunkHeight", c_ushort),
    ("nLostPacket", c_uint),
    ("nReserved", c_uint * 39),
]
MV_FRAME_OUT_INFO_EX = _MV_FRAME_OUT_INFO_EX_


# CameraParams.h 176
class _MV_DISPLAY_FRAME_INFO_(Structure):
    pass


_MV_DISPLAY_FRAME_INFO_._fields_ = [
    # CameraParams.h 176
    ("hWnd", c_void_p),
    ("pData", POINTER(c_ubyte)),
    ("nDataLen", c_uint),
    ("nWidth", c_ushort),
    ("nHeight", c_ushort),
    ("enPixelType", MvGvspPixelType),
    ("nRes", c_uint * 4),
]
MV_DISPLAY_FRAME_INFO = _MV_DISPLAY_FRAME_INFO_

# values for enumeration 'MV_SAVE_IAMGE_TYPE'
MV_SAVE_IAMGE_TYPE = c_int  # enum


# CameraParams.h 198
class _MV_SAVE_IMAGE_PARAM_T_(Structure):
    pass


_MV_SAVE_IMAGE_PARAM_T_._fields_ = [
    # CameraParams.h 198
    ("pData", POINTER(c_ubyte)),
    ("nDataLen", c_uint),
    ("enPixelType", MvGvspPixelType),
    ("nWidth", c_ushort),
    ("nHeight", c_ushort),
    ("pImageBuffer", POINTER(c_ubyte)),
    ("nImageLen", c_uint),
    ("nBufferSize", c_uint),
    ("enImageType", MV_SAVE_IAMGE_TYPE),
]
MV_SAVE_IMAGE_PARAM = _MV_SAVE_IMAGE_PARAM_T_


# CameraParams.h 214
class _MV_SAVE_IMAGE_PARAM_T_EX_(Structure):
    pass


_MV_SAVE_IMAGE_PARAM_T_EX_._fields_ = [
    # CameraParams.h 214
    ("pData", POINTER(c_ubyte)),
    ("nDataLen", c_uint),
    ("enPixelType", MvGvspPixelType),
    ("nWidth", c_ushort),
    ("nHeight", c_ushort),
    ("pImageBuffer", POINTER(c_ubyte)),
    ("nImageLen", c_uint),
    ("nBufferSize", c_uint),
    ("enImageType", MV_SAVE_IAMGE_TYPE),
    ("nJpgQuality", c_uint),
    ("iMethodValue", c_uint),
    ("nReserved", c_uint * 3),
]
MV_SAVE_IMAGE_PARAM_EX = _MV_SAVE_IMAGE_PARAM_T_EX_


# CameraParams.h 236
class _MV_PIXEL_CONVERT_PARAM_T_(Structure):
    pass


_MV_PIXEL_CONVERT_PARAM_T_._fields_ = [
    # CameraParams.h 236
    ("nWidth", c_ushort),
    ("nHeight", c_ushort),
    ("enSrcPixelType", MvGvspPixelType),
    ("pSrcData", POINTER(c_ubyte)),
    ("nSrcDataLen", c_uint),
    ("enDstPixelType", MvGvspPixelType),
    ("pDstBuffer", POINTER(c_ubyte)),
    ("nDstLen", c_uint),
    ("nDstBufferSize", c_uint),
    ("nRes", c_uint * 4),
]
MV_CC_PIXEL_CONVERT_PARAM = _MV_PIXEL_CONVERT_PARAM_T_

# values for enumeration '_MV_CAM_ACQUISITION_MODE_'
_MV_CAM_ACQUISITION_MODE_ = c_int  # enum
MV_CAM_ACQUISITION_MODE = _MV_CAM_ACQUISITION_MODE_

# values for enumeration '_MV_CAM_GAIN_MODE_'
_MV_CAM_GAIN_MODE_ = c_int  # enum
MV_CAM_GAIN_MODE = _MV_CAM_GAIN_MODE_

# values for enumeration '_MV_CAM_EXPOSURE_MODE_'
_MV_CAM_EXPOSURE_MODE_ = c_int  # enum
MV_CAM_EXPOSURE_MODE = _MV_CAM_EXPOSURE_MODE_

# values for enumeration '_MV_CAM_EXPOSURE_AUTO_MODE_'
_MV_CAM_EXPOSURE_AUTO_MODE_ = c_int  # enum
MV_CAM_EXPOSURE_AUTO_MODE = _MV_CAM_EXPOSURE_AUTO_MODE_

# values for enumeration '_MV_CAM_TRIGGER_MODE_'
_MV_CAM_TRIGGER_MODE_ = c_int  # enum
MV_CAM_TRIGGER_MODE = _MV_CAM_TRIGGER_MODE_

# values for enumeration '_MV_CAM_GAMMA_SELECTOR_'
_MV_CAM_GAMMA_SELECTOR_ = c_int  # enum
MV_CAM_GAMMA_SELECTOR = _MV_CAM_GAMMA_SELECTOR_

# values for enumeration '_MV_CAM_BALANCEWHITE_AUTO_'
_MV_CAM_BALANCEWHITE_AUTO_ = c_int  # enum
MV_CAM_BALANCEWHITE_AUTO = _MV_CAM_BALANCEWHITE_AUTO_

# values for enumeration '_MV_CAM_TRIGGER_SOURCE_'
_MV_CAM_TRIGGER_SOURCE_ = c_int  # enum
MV_CAM_TRIGGER_SOURCE = _MV_CAM_TRIGGER_SOURCE_

# values for enumeration '_MV_GIGE_TRANSMISSION_TYPE_'
_MV_GIGE_TRANSMISSION_TYPE_ = c_int  # enum
MV_GIGE_TRANSMISSION_TYPE = _MV_GIGE_TRANSMISSION_TYPE_


# CameraParams.h 377
class _MV_ALL_MATCH_INFO_(Structure):
    pass


_MV_ALL_MATCH_INFO_._fields_ = [
    # CameraParams.h 377
    ("nType", c_uint),
    ("pInfo", c_void_p),
    ("nInfoSize", c_uint),
]
MV_ALL_MATCH_INFO = _MV_ALL_MATCH_INFO_


# CameraParams.h 387
class _MV_MATCH_INFO_NET_DETECT_(Structure):
    pass


_MV_MATCH_INFO_NET_DETECT_._fields_ = [
    # CameraParams.h 387
    ("nReviceDataSize", int64_t),
    ("nLostPacketCount", int64_t),
    ("nLostFrameCount", c_uint),
    ("nReserved", c_uint * 5),
]
MV_MATCH_INFO_NET_DETECT = _MV_MATCH_INFO_NET_DETECT_


# CameraParams.h 396
class _MV_MATCH_INFO_USB_DETECT_(Structure):
    pass


_MV_MATCH_INFO_USB_DETECT_._fields_ = [
    # CameraParams.h 396
    ("nReviceDataSize", int64_t),
    ("nRevicedFrameCount", c_uint),
    ("nErrorFrameCount", c_uint),
    ("nReserved", c_uint * 2),
]
MV_MATCH_INFO_USB_DETECT = _MV_MATCH_INFO_USB_DETECT_


# CameraParams.h 404
class _MV_IMAGE_BASIC_INFO_(Structure):
    pass


_MV_IMAGE_BASIC_INFO_._fields_ = [
    # CameraParams.h 404
    ("nWidthValue", c_ushort),
    ("nWidthMin", c_ushort),
    ("nWidthMax", c_uint),
    ("nWidthInc", c_uint),
    ("nHeightValue", c_uint),
    ("nHeightMin", c_uint),
    ("nHeightMax", c_uint),
    ("nHeightInc", c_uint),
    ("fFrameRateValue", c_float),
    ("fFrameRateMin", c_float),
    ("fFrameRateMax", c_float),
    ("enPixelType", c_uint),
    ("nSupportedPixelFmtNum", c_uint),
    ("enPixelList", c_uint * 64),
    ("nReserved", c_uint * 8),
]
MV_IMAGE_BASIC_INFO = _MV_IMAGE_BASIC_INFO_

# values for enumeration 'MV_XML_InterfaceType'
MV_XML_InterfaceType = c_int  # enum

# values for enumeration 'MV_XML_AccessMode'
MV_XML_AccessMode = c_int  # enum

# values for enumeration 'MV_XML_Visibility'
MV_XML_Visibility = c_int  # enum


# CameraParams.h 500
class _MV_EVENT_OUT_INFO_(Structure):
    pass


_MV_EVENT_OUT_INFO_._fields_ = [
    # CameraParams.h 500
    ("EventName", c_char * 128),
    ("nEventID", c_ushort),
    ("nStreamChannel", c_ushort),
    ("nBlockIdHigh", c_uint),
    ("nBlockIdLow", c_uint),
    ("nTimestampHigh", c_uint),
    ("nTimestampLow", c_uint),
    ("pEventData", c_void_p),
    ("nEventDataSize", c_uint),
    ("nReserved", c_uint * 16),
]
MV_EVENT_OUT_INFO = _MV_EVENT_OUT_INFO_


# CameraParams.h 520
class _MV_CC_FILE_ACCESS_T(Structure):
    pass


_MV_CC_FILE_ACCESS_T._fields_ = [
    # CameraParams.h 520
    ("pUserFileName", STRING),
    ("pDevFileName", STRING),
    ("nReserved", c_uint * 32),
]
MV_CC_FILE_ACCESS = _MV_CC_FILE_ACCESS_T


# CameraParams.h 529
class _MV_CC_FILE_ACCESS_PROGRESS_T(Structure):
    pass


_MV_CC_FILE_ACCESS_PROGRESS_T._fields_ = [
    # CameraParams.h 529
    ("nCompleted", int64_t),
    ("nTotal", int64_t),
    ("nReserved", c_uint * 8),
]
MV_CC_FILE_ACCESS_PROGRESS = _MV_CC_FILE_ACCESS_PROGRESS_T


# CameraParams.h 538
class _MV_TRANSMISSION_TYPE_T(Structure):
    pass


_MV_TRANSMISSION_TYPE_T._fields_ = [
    # CameraParams.h 538
    ("enTransmissionType", MV_GIGE_TRANSMISSION_TYPE),
    ("nDestIp", c_uint),
    ("nDestPort", c_ushort),
    ("nReserved", c_uint * 32),
]
MV_TRANSMISSION_TYPE = _MV_TRANSMISSION_TYPE_T


# CameraParams.h 548
class _MV_XML_NODE_FEATURE_(Structure):
    pass


_MV_XML_NODE_FEATURE_._fields_ = [
    # CameraParams.h 548
    ("enType", MV_XML_InterfaceType),
    ("enVisivility", MV_XML_Visibility),
    ("strDescription", c_char * 512),
    ("strDisplayName", c_char * 64),
    ("strName", c_char * 64),
    ("strToolTip", c_char * 512),
    ("nReserved", c_uint * 4),
]
MV_XML_NODE_FEATURE = _MV_XML_NODE_FEATURE_


# CameraParams.h 561
class _MV_XML_NODES_LIST_(Structure):
    pass


_MV_XML_NODES_LIST_._fields_ = [
    # CameraParams.h 561
    ("nNodeNum", c_uint),
    ("stNodes", MV_XML_NODE_FEATURE * 128),
]
MV_XML_NODES_LIST = _MV_XML_NODES_LIST_


# CameraParams.h 569
class _MV_XML_FEATURE_Value_(Structure):
    pass


_MV_XML_FEATURE_Value_._fields_ = [
    # CameraParams.h 569
    ("enType", MV_XML_InterfaceType),
    ("strDescription", c_char * 512),
    ("strDisplayName", c_char * 64),
    ("strName", c_char * 64),
    ("strToolTip", c_char * 512),
    ("nReserved", c_uint * 4),
]
MV_XML_FEATURE_Value = _MV_XML_FEATURE_Value_


# CameraParams.h 579
class _MV_XML_FEATURE_Base_(Structure):
    pass


_MV_XML_FEATURE_Base_._fields_ = [
    # CameraParams.h 579
    ("enAccessMode", MV_XML_AccessMode),
]
MV_XML_FEATURE_Base = _MV_XML_FEATURE_Base_


# CameraParams.h 584
class _MV_XML_FEATURE_Integer_(Structure):
    pass


_MV_XML_FEATURE_Integer_._fields_ = [
    # CameraParams.h 584
    ("strName", c_char * 64),
    ("strDisplayName", c_char * 64),
    ("strDescription", c_char * 512),
    ("strToolTip", c_char * 512),
    ("enVisivility", MV_XML_Visibility),
    ("enAccessMode", MV_XML_AccessMode),
    ("bIsLocked", c_int),
    ("nValue", int64_t),
    ("nMinValue", int64_t),
    ("nMaxValue", int64_t),
    ("nIncrement", int64_t),
    ("nReserved", c_uint * 4),
]
MV_XML_FEATURE_Integer = _MV_XML_FEATURE_Integer_


# CameraParams.h 603
class _MV_XML_FEATURE_Boolean_(Structure):
    pass


_MV_XML_FEATURE_Boolean_._fields_ = [
    # CameraParams.h 603
    ("strName", c_char * 64),
    ("strDisplayName", c_char * 64),
    ("strDescription", c_char * 512),
    ("strToolTip", c_char * 512),
    ("enVisivility", MV_XML_Visibility),
    ("enAccessMode", MV_XML_AccessMode),
    ("bIsLocked", c_int),
    ("bValue", c_bool),
    ("nReserved", c_uint * 4),
]
MV_XML_FEATURE_Boolean = _MV_XML_FEATURE_Boolean_


# CameraParams.h 618
class _MV_XML_FEATURE_Command_(Structure):
    pass


_MV_XML_FEATURE_Command_._fields_ = [
    # CameraParams.h 618
    ("strName", c_char * 64),
    ("strDisplayName", c_char * 64),
    ("strDescription", c_char * 512),
    ("strToolTip", c_char * 512),
    ("enVisivility", MV_XML_Visibility),
    ("enAccessMode", MV_XML_AccessMode),
    ("bIsLocked", c_int),
    ("nReserved", c_uint * 4),
]
MV_XML_FEATURE_Command = _MV_XML_FEATURE_Command_


# CameraParams.h 632
class _MV_XML_FEATURE_Float_(Structure):
    pass


_MV_XML_FEATURE_Float_._fields_ = [
    # CameraParams.h 632
    ("strName", c_char * 64),
    ("strDisplayName", c_char * 64),
    ("strDescription", c_char * 512),
    ("strToolTip", c_char * 512),
    ("enVisivility", MV_XML_Visibility),
    ("enAccessMode", MV_XML_AccessMode),
    ("bIsLocked", c_int),
    ("dfValue", c_double),
    ("dfMinValue", c_double),
    ("dfMaxValue", c_double),
    ("dfIncrement", c_double),
    ("nReserved", c_uint * 4),
]
MV_XML_FEATURE_Float = _MV_XML_FEATURE_Float_


# CameraParams.h 650
class _MV_XML_FEATURE_String_(Structure):
    pass


_MV_XML_FEATURE_String_._fields_ = [
    # CameraParams.h 650
    ("strName", c_char * 64),
    ("strDisplayName", c_char * 64),
    ("strDescription", c_char * 512),
    ("strToolTip", c_char * 512),
    ("enVisivility", MV_XML_Visibility),
    ("enAccessMode", MV_XML_AccessMode),
    ("bIsLocked", c_int),
    ("strValue", c_char * 64),
    ("nReserved", c_uint * 4),
]
MV_XML_FEATURE_String = _MV_XML_FEATURE_String_


# CameraParams.h 665
class _MV_XML_FEATURE_Register_(Structure):
    pass


_MV_XML_FEATURE_Register_._fields_ = [
    # CameraParams.h 665
    ("strName", c_char * 64),
    ("strDisplayName", c_char * 64),
    ("strDescription", c_char * 512),
    ("strToolTip", c_char * 512),
    ("enVisivility", MV_XML_Visibility),
    ("enAccessMode", MV_XML_AccessMode),
    ("bIsLocked", c_int),
    ("nAddrValue", int64_t),
    ("nReserved", c_uint * 4),
]
MV_XML_FEATURE_Register = _MV_XML_FEATURE_Register_


# CameraParams.h 680
class _MV_XML_FEATURE_Category_(Structure):
    pass


_MV_XML_FEATURE_Category_._fields_ = [
    # CameraParams.h 680
    ("strDescription", c_char * 512),
    ("strDisplayName", c_char * 64),
    ("strName", c_char * 64),
    ("strToolTip", c_char * 512),
    ("enVisivility", MV_XML_Visibility),
    ("nReserved", c_uint * 4),
]
MV_XML_FEATURE_Category = _MV_XML_FEATURE_Category_


# CameraParams.h 692
class _MV_XML_FEATURE_EnumEntry_(Structure):
    pass


_MV_XML_FEATURE_EnumEntry_._fields_ = [
    # CameraParams.h 692
    ("strName", c_char * 64),
    ("strDisplayName", c_char * 64),
    ("strDescription", c_char * 512),
    ("strToolTip", c_char * 512),
    ("bIsImplemented", c_int),
    ("nParentsNum", c_int),
    ("stParentsList", MV_XML_NODE_FEATURE * 8),
    ("enVisivility", MV_XML_Visibility),
    ("nValue", int64_t),
    ("enAccessMode", MV_XML_AccessMode),
    ("bIsLocked", c_int),
    ("nReserved", c_int * 8),
]
MV_XML_FEATURE_EnumEntry = _MV_XML_FEATURE_EnumEntry_


# CameraParams.h 711
class _MV_XML_FEATURE_Enumeration_(Structure):
    pass


_MV_XML_FEATURE_Enumeration_._fields_ = [
    # CameraParams.h 711
    ("enVisivility", MV_XML_Visibility),
    ("strDescription", c_char * 512),
    ("strDisplayName", c_char * 64),
    ("strName", c_char * 64),
    ("strToolTip", c_char * 512),
    ("nSymbolicNum", c_int),
    ("strCurrentSymbolic", c_char * 64),
    ("strSymbolic", c_char * 64 * 64),
    ("enAccessMode", MV_XML_AccessMode),
    ("bIsLocked", c_int),
    ("nValue", int64_t),
    ("nReserved", c_uint * 4),
]
MV_XML_FEATURE_Enumeration = _MV_XML_FEATURE_Enumeration_


# CameraParams.h 730
class _MV_XML_FEATURE_Port_(Structure):
    pass


_MV_XML_FEATURE_Port_._fields_ = [
    # CameraParams.h 730
    ("enVisivility", MV_XML_Visibility),
    ("strDescription", c_char * 512),
    ("strDisplayName", c_char * 64),
    ("strName", c_char * 64),
    ("strToolTip", c_char * 512),
    ("enAccessMode", MV_XML_AccessMode),
    ("bIsLocked", c_int),
    ("nReserved", c_uint * 4),
]
MV_XML_FEATURE_Port = _MV_XML_FEATURE_Port_


# CameraParams.h 746
class _MV_XML_CAMERA_FEATURE_(Structure):
    pass


# CameraParams.h 749
class N23_MV_XML_CAMERA_FEATURE_3DOT_1E(Union):
    pass


N23_MV_XML_CAMERA_FEATURE_3DOT_1E._fields_ = [
    # CameraParams.h 749
    ("stIntegerFeature", MV_XML_FEATURE_Integer),
    ("stFloatFeature", MV_XML_FEATURE_Float),
    ("stEnumerationFeature", MV_XML_FEATURE_Enumeration),
    ("stStringFeature", MV_XML_FEATURE_String),
]
_MV_XML_CAMERA_FEATURE_._fields_ = [
    # CameraParams.h 746
    ("enType", MV_XML_InterfaceType),
    ("SpecialFeature", N23_MV_XML_CAMERA_FEATURE_3DOT_1E),
]
MV_XML_CAMERA_FEATURE = _MV_XML_CAMERA_FEATURE_


# CameraParams.h 760
class _MVCC_ENUMVALUE_T(Structure):
    pass


_MVCC_ENUMVALUE_T._fields_ = [
    # CameraParams.h 760
    ("nCurValue", c_uint),
    ("nSupportedNum", c_uint),
    ("nSupportValue", c_uint * 64),
    ("nReserved", c_uint * 4),
]
MVCC_ENUMVALUE = _MVCC_ENUMVALUE_T


# CameraParams.h 769
class _MVCC_INTVALUE_T(Structure):
    pass


_MVCC_INTVALUE_T._fields_ = [
    # CameraParams.h 769
    ("nCurValue", c_uint),
    ("nMax", c_uint),
    ("nMin", c_uint),
    ("nInc", c_uint),
    ("nReserved", c_uint * 4),
]
MVCC_INTVALUE = _MVCC_INTVALUE_T


# CameraParams.h 779
class _MVCC_INTVALUE_EX_T(Structure):
    pass


_MVCC_INTVALUE_EX_T._fields_ = [
    # CameraParams.h 779
    ("nCurValue", int64_t),
    ("nMax", int64_t),
    ("nMin", int64_t),
    ("nInc", int64_t),
    ("nReserved", c_uint * 16),
]
MVCC_INTVALUE_EX = _MVCC_INTVALUE_EX_T


# CameraParams.h 789
class _MVCC_FLOATVALUE_T(Structure):
    pass


_MVCC_FLOATVALUE_T._fields_ = [
    # CameraParams.h 789
    ("fCurValue", c_float),
    ("fMax", c_float),
    ("fMin", c_float),
    ("nReserved", c_uint * 4),
]
MVCC_FLOATVALUE = _MVCC_FLOATVALUE_T


# CameraParams.h 798
class _MVCC_STRINGVALUE_T(Structure):
    pass


_MVCC_STRINGVALUE_T._fields_ = [
    # CameraParams.h 798
    ("chCurValue", c_char * 256),
    ("nMaxLength", int64_t),
    ("nReserved", c_uint * 2),
]
MVCC_STRINGVALUE = _MVCC_STRINGVALUE_T
__all__ = [
    "_MV_ALL_MATCH_INFO_",
    "_MV_XML_FEATURE_Integer_",
    "MV_CC_FILE_ACCESS_PROGRESS",
    "N19_MV_CC_DEVICE_INFO_3DOT_0E",
    "MV_CAM_EXPOSURE_AUTO_MODE",
    "PixelType_Gvsp_BayerRG8",
    "MV_CAM_GAIN_MODE",
    "MV_GIGE_TRANSTYPE_UNICAST_WITHOUT_RECV",
    "PixelType_Gvsp_BayerGB12_Packed",
    "MV_TRIGGER_SOURCE_LINE0",
    "MV_TRIGGER_SOURCE_LINE1",
    "MV_TRIGGER_SOURCE_LINE2",
    "MV_TRIGGER_SOURCE_LINE3",
    "AM_CycleDetect",
    "PixelType_Gvsp_YCBCR422_8",
    "IFT_IFloat",
    "MV_TRANSMISSION_TYPE",
    "_MV_XML_FEATURE_Command_",
    "_MV_XML_FEATURE_String_",
    "_MV_CAM_TRIGGER_SOURCE_",
    "PixelType_Gvsp_COORD3D_DEPTH_PLUS_MASK",
    "PixelType_Gvsp_RGB10V1_Packed",
    "PixelType_Gvsp_RGB10_Packed",
    "PixelType_Gvsp_RGB8_Planar",
    "AM_RO",
    "IFT_IPort",
    "uint_least16_t",
    "_MV_FRAME_OUT_INFO_EX_",
    "_MV_TRANSMISSION_TYPE_T",
    "MV_SAVE_IMAGE_PARAM_EX",
    "AM_RW",
    "MV_XML_InterfaceType",
    "_MV_XML_CAMERA_FEATURE_",
    "intptr_t",
    "uint_least64_t",
    "V_Guru",
    "_MV_CAM_TRIGGER_MODE_",
    "MV_CAM_EXPOSURE_MODE",
    "int_least32_t",
    "MV_GIGE_TRANSTYPE_SUBNETBROADCAST",
    "_MV_XML_FEATURE_Boolean_",
    "MV_BALANCEWHITE_AUTO_CONTINUOUS",
    "MV_XML_NODE_FEATURE",
    "_MV_FRAME_OUT_INFO_",
    "MV_ALL_MATCH_INFO",
    "_MV_XML_FEATURE_EnumEntry_",
    "_MV_PIXEL_CONVERT_PARAM_T_",
    "PixelType_Gvsp_RGB16_Packed",
    "PixelType_Gvsp_RGBA8_Packed",
    "PixelType_Gvsp_RGB8_Packed",
    "MV_ACQ_MODE_SINGLE",
    "MV_TRIGGER_MODE_ON",
    "_MV_XML_FEATURE_Base_",
    "PixelType_Gvsp_Mono8_Signed",
    "int_least16_t",
    "MV_GIGE_TRANSTYPE_LIMITEDBROADCAST",
    "int_fast32_t",
    "_MV_CAM_GAIN_MODE_",
    "uint_fast16_t",
    "_MV_DISPLAY_FRAME_INFO_",
    "IFT_ICommand",
    "PixelType_Gvsp_BayerBG10_Packed",
    "_MV_CAM_ACQUISITION_MODE_",
    "PixelType_Gvsp_YCBCR709_422_8_CBYCRY",
    "V_Beginner",
    "PixelType_Gvsp_BayerBG16",
    "MV_GIGE_TRANSTYPE_MULTICAST_WITHOUT_RECV",
    "MVCC_INTVALUE",
    "_MV_XML_FEATURE_Port_",
    "_MV_GIGE_TRANSMISSION_TYPE_",
    "MV_GIGE_TRANSMISSION_TYPE",
    "PixelType_Gvsp_YUV411_Packed",
    "_MV_XML_FEATURE_Register_",
    "MV_EXPOSURE_MODE_TIMED",
    "intmax_t",
    "PixelType_Gvsp_BayerBG12_Packed",
    "int16_t",
    "MV_DISPLAY_FRAME_INFO",
    "MV_XML_FEATURE_Integer",
    "_MV_CC_FILE_ACCESS_PROGRESS_T",
    "MV_XML_Visibility",
    "IFT_IEnumEntry",
    "int_fast64_t",
    "PixelType_Gvsp_BayerRG12",
    "PixelType_Gvsp_BayerRG10",
    "MV_XML_AccessMode",
    "V_Expert",
    "MV_GAIN_MODE_ONCE",
    "IFT_IInteger",
    "PixelType_Gvsp_YCBCR709_411_8_CBYYCRYY",
    "MV_CAM_BALANCEWHITE_AUTO",
    "int_least8_t",
    "IFT_IBase",
    "MV_XML_NODES_LIST",
    "MV_TRIGGER_MODE_OFF",
    "MV_Image_Bmp",
    "MV_XML_FEATURE_String",
    "MV_CC_FILE_ACCESS",
    "_MV_CAM_EXPOSURE_AUTO_MODE_",
    "PixelType_Gvsp_Coord3D_AC32f",
    "uint_least8_t",
    "PixelType_Gvsp_BayerRG12_Packed",
    "_MV_XML_FEATURE_Float_",
    "_MV_XML_NODE_FEATURE_",
    "MV_XML_FEATURE_Float",
    "PixelType_Gvsp_BGR12_Packed",
    "PixelType_Gvsp_YUV422_Packed",
    "MV_SAVE_IMAGE_PARAM",
    "MV_EVENT_OUT_INFO",
    "IFT_IEnumeration",
    "uint64_t",
    "uint8_t",
    "PixelType_Gvsp_RGB12_Planar",
    "V_Undefined",
    "_MVCC_STRINGVALUE_T",
    "PixelType_Gvsp_YCBCR709_422_8",
    "MV_CAM_TRIGGER_MODE",
    "PixelType_Gvsp_BayerRG10_Packed",
    "N23_MV_XML_CAMERA_FEATURE_3DOT_1E",
    "uint16_t",
    "uint_fast8_t",
    "_MV_SAVE_IMAGE_PARAM_T_",
    "PixelType_Gvsp_BGR8_Packed",
    "_MVCC_ENUMVALUE_T",
    "_MV_MATCH_INFO_USB_DETECT_",
    "MV_XML_FEATURE_Category",
    "PixelType_Gvsp_Jpeg",
    "int32_t",
    "PixelType_Gvsp_BayerGR10_Packed",
    "MV_XML_FEATURE_EnumEntry",
    "_MV_CC_DEVICE_INFO_",
    "PixelType_Gvsp_BayerBG12",
    "PixelType_Gvsp_BayerBG10",
    "IFT_IBoolean",
    "PixelType_Gvsp_Mono12",
    "MV_MATCH_INFO_USB_DETECT",
    "PixelType_Gvsp_RGB16_Planar",
    "MVCC_ENUMVALUE",
    "IFT_IString",
    "PixelType_Gvsp_BayerRG16",
    "_MV_XML_FEATURE_Value_",
    "MV_ACQ_MODE_CONTINUOUS",
    "MV_TRIGGER_SOURCE_FrequencyConverter",
    "PixelType_Gvsp_Mono4p",
    "MV_TRIGGER_SOURCE_COUNTER0",
    "MV_GAIN_MODE_OFF",
    "MV_Image_Png",
    "_MV_CC_DEVICE_INFO_LIST_",
    "MV_GIGE_DEVICE_INFO",
    "_MV_SAVE_IMAGE_PARAM_T_EX_",
    "PixelType_Gvsp_Mono8",
    "uint_least32_t",
    "PixelType_Gvsp_BayerGR16",
    "MV_FRAME_OUT_INFO",
    "PixelType_Gvsp_BayerGR10",
    "PixelType_Gvsp_BGRA8_Packed",
    "PixelType_Gvsp_BayerGR12",
    "_MVCC_INTVALUE_EX_T",
    "uintptr_t",
    "PixelType_Gvsp_Mono12_Packed",
    "MVCC_FLOATVALUE",
    "MV_GIGE_TRANSTYPE_CAMERADEFINED",
    "_MV_XML_NODES_LIST_",
    "MV_NETTRANS_INFO",
    "IFT_IRegister",
    "AM_NA",
    "MV_GIGE_TRANSTYPE_UNICAST",
    "int8_t",
    "_MV_GIGE_DEVICE_INFO_",
    "IFT_IValue",
    "IFT_ICategory",
    "int_fast8_t",
    "PixelType_Gvsp_YCBCR709_8_CBYCR",
    "MV_XML_FEATURE_Enumeration",
    "MV_GAMMA_SELECTOR_SRGB",
    "PixelType_Gvsp_YCBCR601_422_8",
    "PixelType_Gvsp_YCBCR411_8_CBYYCRYY",
    "int_least64_t",
    "MV_EXPOSURE_AUTO_MODE_OFF",
    "MV_CC_PIXEL_CONVERT_PARAM",
    "MV_EXPOSURE_AUTO_MODE_CONTINUOUS",
    "MV_CAM_ACQUISITION_MODE",
    "_MVCC_INTVALUE_T",
    "MV_XML_FEATURE_Value",
    "AM_Undefined",
    "MV_MATCH_INFO_NET_DETECT",
    "PixelType_Gvsp_Undefined",
    "_MV_CC_FILE_ACCESS_T",
    "AM_NI",
    "PixelType_Gvsp_BayerGR12_Packed",
    "V_Invisible",
    "PixelType_Gvsp_YCBCR601_411_8_CBYYCRYY",
    "PixelType_Gvsp_RGB10_Planar",
    "MV_CAM_GAMMA_SELECTOR",
    "MV_TRIGGER_SOURCE_SOFTWARE",
    "PixelType_Gvsp_BayerGB16",
    "MV_BALANCEWHITE_AUTO_ONCE",
    "PixelType_Gvsp_BayerGB10",
    "uintmax_t",
    "PixelType_Gvsp_BayerGB12",
    "int_fast16_t",
    "_MV_CAM_EXPOSURE_MODE_",
    "MV_Image_Tif",
    "PixelType_Gvsp_BGR565_Packed",
    "MV_BALANCEWHITE_AUTO_OFF",
    "int64_t",
    "MV_Image_Undefined",
    "_MV_NETTRANS_INFO_",
    "MV_GAIN_MODE_CONTINUOUS",
    "PixelType_Gvsp_Mono1p",
    "uint_fast32_t",
    "MV_CAM_TRIGGER_SOURCE",
    "PixelType_Gvsp_YUV444_Packed",
    "MV_Image_Jpeg",
    "_MVCC_FLOATVALUE_T",
    "MV_XML_FEATURE_Port",
    "PixelType_Gvsp_YUV422_YUYV_Packed",
    "MV_FRAME_OUT_INFO_EX",
    "_MV_IMAGE_BASIC_INFO_",
    "_MV_CAM_BALANCEWHITE_AUTO_",
    "MV_XML_FEATURE_Base",
    "_MV_USB3_DEVICE_INFO_",
    "PixelType_Gvsp_BayerBG8",
    "MVCC_INTVALUE_EX",
    "MV_XML_FEATURE_Register",
    "AM_WO",
    "MV_GIGE_TRANSTYPE_UNICAST_DEFINED_PORT",
    "_MV_XML_FEATURE_Enumeration_",
    "_MV_MATCH_INFO_NET_DETECT_",
    "MV_SAVE_IAMGE_TYPE",
    "MV_EXPOSURE_AUTO_MODE_ONCE",
    "PixelType_Gvsp_BGR10_Packed",
    "PixelType_Gvsp_BayerGB10_Packed",
    "PixelType_Gvsp_Coord3D_ABC32f_Planar",
    "MV_GIGE_TRANSTYPE_MULTICAST",
    "MV_XML_CAMERA_FEATURE",
    "PixelType_Gvsp_Coord3D_ABC32f",
    "MVCC_STRINGVALUE",
    "MV_CC_DEVICE_INFO",
    "PixelType_Gvsp_YCBCR422_8_CBYCRY",
    "PixelType_Gvsp_RGB12_Packed",
    "PixelType_Gvsp_BayerGR8",
    "MvGvspPixelType",
    "PixelType_Gvsp_Mono10",
    "MV_IMAGE_BASIC_INFO",
    "PixelType_Gvsp_Mono16",
    "PixelType_Gvsp_Mono2p",
    "PixelType_Gvsp_Mono14",
    "PixelType_Gvsp_RGB10V2_Packed",
    "MV_CC_DEVICE_INFO_LIST",
    "PixelType_Gvsp_RGB12V1_Packed",
    "PixelType_Gvsp_Mono10_Packed",
    "uint_fast64_t",
    "PixelType_Gvsp_YCBCR601_8_CBYCR",
    "PixelType_Gvsp_BayerGB8",
    "_MV_XML_FEATURE_Category_",
    "MV_EXPOSURE_MODE_TRIGGER_WIDTH",
    "PixelType_Gvsp_YCBCR8_CBYCR",
    "MV_XML_FEATURE_Boolean",
    "MV_GAMMA_SELECTOR_USER",
    "PixelType_Gvsp_RGB565_Packed",
    "uint32_t",
    "MV_XML_FEATURE_Command",
    "_MV_CAM_GAMMA_SELECTOR_",
    "MV_ACQ_MODE_MUTLI",
    "PixelType_Gvsp_YCBCR601_422_8_CBYCRY",
    "MV_USB3_DEVICE_INFO",
    "_MV_EVENT_OUT_INFO_",
]



"""Camera wrapper"""
from typing import Any, Optional, Callable, Union
import logging
from dataclasses import dataclass, field
import functools
import ctypes

from .hiklibs.MvCameraControl_class import MvCamera
from .hiklibs.CameraParams_const import MV_ACCESS_Exclusive
from .hiklibs.CameraParams_header import MVCC_INTVALUE, MVCC_FLOATVALUE
from .types import DeviceInfo, Frame, FrameCallbackType, DeviceTransport
from .errors import (
    HIKException,
    HandleCreateError,
    HandleDestroyError,
    mv_errstr,
    CamCloseError,
    CamOpenError,
    CamClosed,
    CamStateError,
    CamSettingsError,
    CamCommandError,
    CamReadError,
)


LOGGER = logging.getLogger(__name__)


@dataclass
class Camera:
    """Wrap the camera pointer to a more pythonic api"""

    info: DeviceInfo = field()
    frame_callback: Optional[Callable[[Frame, "Camera"], None]] = field(default=None)

    raise_on_setting_failure: bool = field(default=True)

    _mvcam: MvCamera = field(init=False, repr=False)
    _open: bool = field(init=False, default=False)
    _grabbing: bool = field(init=False, default=False)
    _mv_callback: Optional[Callable[..., None]] = field(init=False, default=None)
    exposure_cache: float = field(init=False, default=0.0)  # the frame info exposure seems to be always 0.0

    def __post_init__(self) -> None:
        """init special properties"""
        self._mvcam = MvCamera()  # type: ignore[no-untyped-call]
        ret = self._mvcam.MV_CC_CreateHandle(self.info.mvcamhandle)  # type: ignore[no-untyped-call]
        if ret != 0:
            raise HandleCreateError(mv_errstr(ret))

    def __del__(self) -> None:
        """Release the handle"""
        if self._open:
            LOGGER.error("Deleting still open camera")
            try:
                self.close()
            except HIKException as exc:
                LOGGER.error("Got {} when closing".format(exc))
        ret = self._mvcam.MV_CC_DestroyHandle()  # type: ignore[no-untyped-call]
        if ret != 0:
            raise HandleDestroyError(mv_errstr(ret))

    @property
    def closed(self) -> bool:
        """Is the camera closed"""
        return not self._open

    def trigger_enable(self, enabled: bool = True) -> bool:
        """Enable or disable trigger"""
        if not self._open:
            raise CamClosed()
        mode = "On"
        if not enabled:
            mode = "Off"
        return self.set_enum("TriggerMode", mode)

    def set_trigger_source(self, sourcename: str) -> bool:
        """Set the trigger source, generally "Software" or "Line0"."""
        return self.set_enum("TriggerSource", sourcename)

    def send_trigger(self) -> None:
        """Send SW trigger to camera"""
        ret = self._mvcam.MV_CC_SetCommandValue("TriggerSoftware")  # type: ignore[no-untyped-call]
        if ret != 0:
            raise CamCommandError(mv_errstr(ret))

    def set_framerate(self, value: float) -> bool:
        """Set the camera framerate"""
        if not self.set_bool("AcquisitionFrameRateEnable", True):
            return False
        return self.set_float("AcquisitionFrameRate", value)

    def optimize_gige_packetsize(self) -> bool:
        """There is a magical helper to determine best packet size, use it and set to cameera"""
        if self.info.transport != DeviceTransport.GIGE:
            LOGGER.error("Not a gige camera")
            return False
        size = self._mvcam.MV_CC_GetOptimalPacketSize()  # type: ignore[no-untyped-call]
        LOGGER.info("Optimal packet size is {}".format(size))
        return self.set_gige_packetsize(size)

    def set_gige_packetsize(self, size: int) -> bool:
        """Set the packet size"""
        if self.info.transport != DeviceTransport.GIGE:
            LOGGER.error("Not a gige camera")
            return False
        return self.set_int("GevSCPSPacketSize", size)

    def set_exposure(self, value: Union[float, int]) -> bool:
        """Set the exposure time and set exposure to timed.
        value on camera is float but it's uSec so floats don't make too much sense"""
        self.set_enum("ExposureMode", "Timed")
        ret = self.set_float("ExposureTime", float(value))
        self.exposure_cache = float(value)
        return ret

    def start(self) -> None:
        """Start grabbing"""
        if not self._open:
            raise CamClosed()
        self._mv_callback = FrameCallbackType(functools.partial(frame_callback, wrapper=self))
        ret = self._mvcam.MV_CC_RegisterImageCallBackEx(self._mv_callback, None)  # type: ignore[no-untyped-call]
        if ret != 0:
            raise CamStateError(mv_errstr(ret))

        ret = self._mvcam.MV_CC_StartGrabbing()  # type: ignore[no-untyped-call]
        if ret != 0:
            raise CamStateError(mv_errstr(ret))
        self._grabbing = True

    def stop(self) -> None:
        """Stop grabbing"""
        if not self._grabbing:
            LOGGER.warning("Not grabbing")
            return
        ret = self._mvcam.MV_CC_StopGrabbing()  # type: ignore[no-untyped-call]
        if ret != 0:
            raise CamStateError(mv_errstr(ret))
        self._grabbing = False

    def open(self) -> None:
        """Open the camera (we only support exclusive mode)"""
        if self._open:
            LOGGER.warning("Already open")
            return
        ret = self._mvcam.MV_CC_OpenDevice(MV_ACCESS_Exclusive, 0)  # type: ignore[no-untyped-call]
        if ret != 0:
            raise CamOpenError(mv_errstr(ret))
        self._open = True
        self.exposure_cache = self.get_float("ExposureTime")

    def close(self) -> None:
        """Close the camera"""
        if not self._open:
            LOGGER.warning("Already closed")
            return
        if self._grabbing:
            LOGGER.error("Closing camera that is grabbing")
            try:
                self.stop()
            except HIKException as exc:
                LOGGER.error("Got {} when stopping".format(exc))
        ret = self._mvcam.MV_CC_CloseDevice()  # type: ignore[no-untyped-call]
        if ret != 0:
            raise CamCloseError(mv_errstr(ret))
        self._open = False

    def set_int(self, varname: str, value: int) -> bool:
        """Set integer value on the camera"""
        ret = self._mvcam.MV_CC_SetIntValue(varname, value)  # type: ignore[no-untyped-call]
        if ret != 0:
            LOGGER.error("Failed to set {}, error {}".format(varname, mv_errstr(ret)))
            if self.raise_on_setting_failure:
                raise CamSettingsError(mv_errstr(ret))
            return False
        return True

    def get_int(self, varname: str) -> int:
        """Get integer value by name from camera"""
        dataptr = MVCC_INTVALUE()
        ctypes.memset(ctypes.byref(dataptr), 0, ctypes.sizeof(MVCC_INTVALUE))
        ret = self._mvcam.MV_CC_GetIntValue(varname, dataptr)  # type: ignore[no-untyped-call]
        if ret != 0:
            LOGGER.error("Failed to read {}, error {}".format(varname, mv_errstr(ret)))
            raise CamReadError(mv_errstr(ret))
        return int(dataptr.nCurValue)

    def set_enum(self, varname: str, value: str) -> bool:
        """Set an enum filed value byt it's string name"""
        ret = self._mvcam.MV_CC_SetEnumValueByString(varname, value)  # type: ignore[no-untyped-call]
        if ret != 0:
            LOGGER.error("Failed to set {}, error {}".format(varname, mv_errstr(ret)))
            if self.raise_on_setting_failure:
                raise CamSettingsError(mv_errstr(ret))
            return False
        return True

    def set_float(self, varname: str, value: float) -> bool:
        """Set float value on the camera"""
        ret = self._mvcam.MV_CC_SetFloatValue(varname, value)  # type: ignore[no-untyped-call]
        if ret != 0:
            LOGGER.error("Failed to set {}, error {}".format(varname, mv_errstr(ret)))
            if self.raise_on_setting_failure:
                raise CamSettingsError(mv_errstr(ret))
            return False
        return True

    def get_float(self, varname: str) -> float:
        """Get float value by name from camera"""
        dataptr = MVCC_FLOATVALUE()
        ctypes.memset(ctypes.byref(dataptr), 0, ctypes.sizeof(MVCC_FLOATVALUE))
        ret = self._mvcam.MV_CC_GetFloatValue(varname, dataptr)  # type: ignore[no-untyped-call]
        if ret != 0:
            LOGGER.error("Failed to read {}, error {}".format(varname, mv_errstr(ret)))
            raise CamReadError(mv_errstr(ret))
        return float(dataptr.fCurValue)

    def set_bool(self, varname: str, value: bool) -> bool:
        """Set boolean value on the camera"""
        ret = self._mvcam.MV_CC_SetBoolValue(varname, value)  # type: ignore[no-untyped-call]
        if ret != 0:
            LOGGER.error("Failed to set {}, error {}".format(varname, mv_errstr(ret)))
            if self.raise_on_setting_failure:
                raise CamSettingsError(mv_errstr(ret))
            return False
        return True


def frame_callback(dataptr: Any, infoptr: Any, userptr: ctypes.c_void_p, *, wrapper: Camera) -> None:
    """Handle a frame from the C grabber loop"""
    _ = userptr
    frame = Frame(infoptr, dataptr)
    if frame.exposure == 0.0:  # if frame does not have exposure overwrite with cached value
        frame.exposure = wrapper.exposure_cache
    if wrapper.frame_callback:
        wrapper.frame_callback(frame, wrapper)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Don't change this file, it was auto generated by Const_file_Generater.py.
# pylint: skip-file

MV_OK = 0x00000000  # < 成功，无错误 | en:Successed, no error
MV_E_HANDLE = 0x80000000  # < 错误或无效的句柄 | en:Error or invalid handle
MV_E_SUPPORT = 0x80000001  # < 不支持的功能 | en:Not supported function
MV_E_BUFOVER = 0x80000002  # < 缓存已满 | en:Cache is full
MV_E_CALLORDER = 0x80000003  # < 函数调用顺序错误 | en:Function calling order error
MV_E_PARAMETER = 0x80000004  # < 错误的参数 | en:Incorrect parameter
MV_E_RESOURCE = 0x80000006  # < 资源申请失败 | en:Applying resource failed
MV_E_NODATA = 0x80000007  # < 无数据 | en:No data
MV_E_PRECONDITION = 0x80000008  # < 前置条件有误，或运行环境已发生变化 | en:Precondition error, or running environment changed
MV_E_VERSION = 0x80000009  # < 版本不匹配 | en:Version mismatches
MV_E_NOENOUGH_BUF = 0x8000000A  # < 传入的内存空间不足 | en:Insufficient memory
MV_E_ABNORMAL_IMAGE = (
    0x8000000B  # < 异常图像，可能是丢包导致图像不完整 | en:Abnormal image, maybe incomplete image because of lost packet
)
MV_E_LOAD_LIBRARY = 0x8000000C  # < 动态导入DLL失败 | en:Load library failed
MV_E_UNKNOW = 0x800000FF  # < 未知的错误 | en:Unknown error
MV_E_GC_GENERIC = 0x80000100  # < 通用错误 | en:General error
MV_E_GC_ARGUMENT = 0x80000101  # < 参数非法 | en:Illegal parameters
MV_E_GC_RANGE = 0x80000102  # < 值超出范围 | en:The value is out of range
MV_E_GC_PROPERTY = 0x80000103  # < 属性 | en:Property
MV_E_GC_RUNTIME = 0x80000104  # < 运行环境有问题 | en:Running environment error
MV_E_GC_LOGICAL = 0x80000105  # < 逻辑错误 | en:Logical error
MV_E_GC_ACCESS = 0x80000106  # < 节点访问条件有误 | en:Node accessing condition error
MV_E_GC_TIMEOUT = 0x80000107  # < 超时 | en:Timeout
MV_E_GC_DYNAMICCAST = 0x80000108  # < 转换异常 | en:Transformation exception
MV_E_GC_UNKNOW = 0x800001FF  # < GenICam未知错误 | en:GenICam unknown error
MV_E_NOT_IMPLEMENTED = 0x80000200  # < 命令不被设备支持 | en:The command is not supported by device
MV_E_INVALID_ADDRESS = 0x80000201  # < 访问的目标地址不存在 | en:The target address being accessed does not exist
MV_E_WRITE_PROTECT = 0x80000202  # < 目标地址不可写 | en:The target address is not writable
MV_E_ACCESS_DENIED = 0x80000203  # < 设备无访问权限 | en:No permission
MV_E_BUSY = 0x80000204  # < 设备忙，或网络断开 | en:Device is busy, or network disconnected
MV_E_PACKET = 0x80000205  # < 网络包数据错误 | en:Network data packet error
MV_E_NETER = 0x80000206  # < 网络相关错误 | en:Network error
MV_E_IP_CONFLICT = 0x80000221  # < 设备IP冲突 | en:Device IP conflict
MV_E_USB_READ = 0x80000300  # < 读usb出错 | en:Reading USB error
MV_E_USB_WRITE = 0x80000301  # < 写usb出错 | en:Writing USB error
MV_E_USB_DEVICE = 0x80000302  # < 设备异常 | en:Device exception
MV_E_USB_GENICAM = 0x80000303  # < GenICam相关错误 | en:GenICam error
MV_E_USB_BANDWIDTH = 0x80000304  # < 带宽不足  该错误码新增 | en:Insufficient bandwidth, this error code is newly added
MV_E_USB_DRIVER = 0x80000305  # < 驱动不匹配或者未装驱动 | en:Driver mismatch or unmounted drive
MV_E_USB_UNKNOW = 0x800003FF  # < USB未知的错误 | en:USB unknown error
MV_E_UPG_FILE_MISMATCH = 0x80000400  # < 升级固件不匹配 | en:Firmware mismatches
MV_E_UPG_LANGUSGE_MISMATCH = 0x80000401  # < 升级固件语言不匹配 | en:Firmware language mismatches
MV_E_UPG_CONFLICT = 0x80000402  # < 升级冲突（设备已经在升级了再次请求升级即返回此错误） | en:Upgrading conflicted (repeated upgrading requests during device upgrade)
MV_E_UPG_INNER_ERR = 0x80000403  # < 升级时相机内部出现错误 | en:Camera internal error during upgrade
MV_E_UPG_UNKNOW = 0x800004FF  # < 升级时未知错误 | en:Unknown error during upgrade



""" Package HIK Robotics camera drivers and some helpers"""
from .types import Frame, DeviceList, DeviceTransport
from .wrapper import Camera

__version__ = "1.0.0"  # NOTE Use `bump2version --config-file patch` to bump versions correctly


__all__ = ["Frame", "Camera", "DeviceList", "__version__", "DeviceTransport"]



"""helpers"""
from typing import Tuple, Iterable


def get_octets(src: int) -> Tuple[int, int, int, int]:
    """Get the octets from an integer, useful for ips etc. returned highest first"""
    nip1 = (src & 0xFF000000) >> 24
    nip2 = (src & 0x00FF0000) >> 16
    nip3 = (src & 0x0000FF00) >> 8
    nip4 = src & 0x000000FF
    return nip1, nip2, nip3, nip4


def get_str(cstr: Iterable[int]) -> str:
    """Convert C string from the camera structs into str"""
    ret = ""
    for val in cstr:
        if val == 0x0:
            break
        ret += chr(val)
    return ret



# -*- coding: utf-8 -*-
# Don't change this file, it was auto generated by Const_file_Generater.py.
# pylint: skip-file

MV_UNKNOW_DEVICE = 0x00000000  # ch:未知设备类型，保留意义 | en:Unknown Device Type, Reserved
MV_GIGE_DEVICE = 0x00000001  # ch:GigE设备 | en:GigE Device
MV_1394_DEVICE = 0x00000002  # ch:1394-a/b 设备 | en:1394-a/b Device
MV_USB_DEVICE = 0x00000004  # ch:USB3.0 设备 | en:USB3.0 Device
MV_CAMERALINK_DEVICE = 0x00000008  # ch:CameraLink设备 | en:CameraLink Device
INFO_MAX_BUFFER_SIZE = 64
MV_MAX_TLS_NUM = 8
MV_MAX_DEVICE_NUM = 256
MV_IP_CFG_STATIC = 0x05000000
MV_IP_CFG_DHCP = 0x06000000
MV_IP_CFG_LLA = 0x04000000
MV_NET_TRANS_DRIVER = 0x00000001
MV_NET_TRANS_SOCKET = 0x00000002
MV_MATCH_TYPE_NET_DETECT = 0x00000001  # ch:网络流量和丢包信息 | en:Network traffic and packet loss information
MV_MATCH_TYPE_USB_DETECT = (
    0x00000002  # ch:host接收到来自U3V设备的字节总数 | en:The total number of bytes host received from U3V device
)
MV_MAX_XML_NODE_NUM_C = 128
MV_MAX_XML_NODE_STRLEN_C = 64
MV_MAX_XML_STRVALUE_STRLEN_C = 64
MV_MAX_XML_DISC_STRLEN_C = 512
MV_MAX_XML_ENTRY_NUM = 10
MV_MAX_XML_PARENTS_NUM = 8
MV_MAX_XML_SYMBOLIC_STRLEN_C = 64
MV_MAX_XML_SYMBOLIC_NUM = 64
MV_EXCEPTION_DEV_DISCONNECT = 0x00008001  # ch:设备断开连接 | en:The device is disconnected
MV_EXCEPTION_VERSION_CHECK = 0x00008002  # ch:SDK与驱动版本不匹配 | en:SDK does not match the driver version
MV_ACCESS_Exclusive = 1
MV_ACCESS_ExclusiveWithSwitch = 2
MV_ACCESS_Control = 3
MV_ACCESS_ControlWithSwitch = 4
MV_ACCESS_ControlSwitchEnable = 5
MV_ACCESS_ControlSwitchEnableWithKey = 6
MV_ACCESS_Monitor = 7
MAX_EVENT_NAME_SIZE = 128  # 相机Event事件名称最大长度 | en:Max length of event name



# -*- coding: utf-8 -*-
# Don't change this file, it was auto generated by Const_file_Generater.py.
# pylint: skip-file

MV_GVSP_PIX_MONO = 0x01000000
MV_GVSP_PIX_RGB = 0x02000000  # deprecated in version 1.1
MV_GVSP_PIX_COLOR = 0x02000000
MV_GVSP_PIX_CUSTOM = 0x80000000
MV_GVSP_PIX_COLOR_MASK = 0xFF000000
MV_GVSP_PIX_EFFECTIVE_PIXEL_SIZE_MASK = 0x00FF0000
MV_GVSP_PIX_EFFECTIVE_PIXEL_SIZE_SHIFT = 16
MV_GVSP_PIX_ID_MASK = 0x0000FFFF
MV_GVSP_PIX_COUNT = 0x46  # next Pixel ID available



"""Errors"""
from typing import Optional

from .hiklibs import MvErrorDefine_const

MV_ERROR_CODES = {
    getattr(MvErrorDefine_const, name): name for name in dir(MvErrorDefine_const) if name.startswith("MV_")
}


class HIKException(Exception):
    """Generic baseclass for catching things"""


class EnumerationError(HIKException, RuntimeError):
    """Failure to enumerate cameras"""


class CamHandleError(HIKException, RuntimeError):
    """Errors related to camera handles"""


class HandleCreateError(CamHandleError):
    """Failure to create a handle for the camera"""


class HandleDestroyError(CamHandleError):
    """Failure to destroy a handle for the camera"""


class CamStateError(HIKException, RuntimeError):
    """Errors related to camera state, like open/close"""


class CamOpenError(CamStateError):
    """Could not open camera"""


class CamCloseError(CamStateError):
    """Could not close camera"""


class CamSettingsError(CamStateError):
    """Could not not set some camera value"""


class CamClosed(CamStateError):
    """Camera is closed and action needs it to be open"""


class InvalidTransport(HIKException, ValueError):
    """A problem with the transport value"""


class FrameError(HIKException, RuntimeError):
    """A problem with frames"""


class CamCommandError(HIKException, RuntimeError):
    """Errors related to sending commands to camera, like SW trigger"""


class CamReadError(HIKException, RuntimeError):
    """Errors related to reading values from the camera"""


def mv_errstr(code: int) -> Optional[str]:
    """Return the constant name that matches the error code"""
    if code not in MV_ERROR_CODES:
        return None
    return MV_ERROR_CODES[code]
